<?php

namespace Framework\Controllers;

use Framework\Foundation\Helper;
use Framework\Foundation\Config;
use Framework\Modules\UserAccess;

class UserController extends BaseController
{

    public function accessRegister($request)
    {
        $formData = !empty($request->json) ? $request->json : [];
        $formData = json_decode(json_encode($formData), true);

        if (empty($formData['user']) || empty($formData['pass']))
        {
            throw new \Exception('Username and password are required!', AJAX_EX_CODE);
        }

        if (UserAccess::userExists($formData['user']))
        {
            return pre_response_array('User already exists', false);
        }

        if (UserAccess::register($formData))
        {
            return pre_response_array('Registered Successfully', true);
        }
        return pre_response_array('Unable to process your request, Please contact support!', false);
    }

    public function accessLogin($request)
    {
        $formData = $request->json;
        $formData = json_decode(json_encode($formData), true);
        $userData = [];
        $loggedIn   = UserAccess::login($formData);
        if($loggedIn && !empty($formData['user'])){
            $userData = UserAccess::userData($formData);
        }
        
        return pre_response_array('Authenticated', $loggedIn, $userData);
    }

    public function accessLogout($request)
    {
        if (!UserAccess::logout())
        {
            throw new \Exception('Unbale to logged you out, please try again later!');
        }

        return pre_response_array('Logged out');
    }

    public function forgetPassword($request)
    {
        try
        {
            $formData      = $request->json;
            $smtpSettingId = Config::get('addons.smtp.config_id');

            if (!$smtpSettingId)
                throw new \Exception('Missing SMTP configuration.');
            if (empty($formData['email']))
                throw new \Exception('Email field cannot be left blank');
            if (!UserAccess::userExists($formData['email']))
                throw new \Exception('Email Not Found');

            $mailData = $this->prepareMailData(1);

            $formData['to_mail']         = array_get($formData, 'email');
            $formData['token']           = array_get($_SESSION, 'twig_tokens', []);
            $formData['reset_link']      = sprintf('%sreset?confirmtoken=%s&t=%s&email=%s',
                    app()->container()->get('funnelBaseUrl'), md5($mailData['domain_name'] . ' ' . $formData['to_mail']), time(), $formData['to_mail']
            );
            $formData                    = array_merge($mailData, $formData);
            $formData['mail_type']       = 66;
            $formData['smtp_setting_id'] = $smtpSettingId;
            $response                    = $this->httpClient->post(Helper::uniApiEndpoint() . 'ajax/send-mail', ['body' => $formData,]);

            $apiResponse      = json_decode($response->getBody()->getContents(), true);
            $response_message = array_get($apiResponse, 'message', 'The request couldn\'t be completed! Please try again later');

            if ($response->getStatusCode() !== 200)
            {
                return pre_response_array($response_message, false, compact('response_message'));
            }

            if (array_get($apiResponse, 'status') == 'success')
            {
                return pre_response_array('Request has been submitted successfully', true, compact('response_message'));
            }
            return pre_response_array($response_message, false, compact('response_message'));
        }
        catch (\Exception $e)
        {
            return pre_response_array($e->getMessage(), false);
        }
    }

    public function resetPassword($request)
    {
        if (empty($_COOKIE['reset_link']))
        {
            return pre_response_array('Link has been expired', false);
        }

        $formData = json_decode(json_encode($request->json), true);

        if (empty($formData['password']) || empty($formData['confirmPassword']))
        {
            return pre_response_array('Enter new password.', false);
        }

        if ($formData['password'] !== $formData['confirmPassword'])
        {
            return pre_response_array('Password and confirm password does not match.', false);
        }

        if (UserAccess::resetPassword(['user' => $_COOKIE['reset_link'], 'pass' => $formData['password']]))
        {

            return pre_response_array('Password has been updated', true);
        }

        return pre_response_array('User not found', false);
    }

    public function postMail($request)
    {
        $apiResponseMessage          = 'The request couldn\'t be completed! Please try again later';
        $formData                    = $request->json;
        $email_template_id           = isset($formData['email_template_id']) ? $formData['email_template_id'] : '';
        $smtp_setting_id             = Config::get('addons.smtp.config_id');
        $formData['smtp_setting_id'] = $smtp_setting_id;
        $is_contact                  = isset($formData['is_contact']) ? $formData['is_contact'] : 0;
        //send all mail request into elminara
        if (!empty($email_template_id) && !empty($smtp_setting_id))
        {
            $formData['to_mail'] = (!isset($formData['to_mail']) && isset($formData['to'])) ? $formData['to'] : (isset($formData['to_mail']) ? $formData['to_mail'] : Config::get('funnel_support_email'));
            $formData['to_mail'] = empty($formData['to_mail']) ? $formData['to'] : $formData['to_mail'];
            $mailData            = $this->prepareMailData(1);
            $mailData['token']   = isset($_SESSION['twig_tokens']) ? $_SESSION['twig_tokens'] : [];
            $formData            = array_merge($mailData, $formData);
            $api                 = Helper::uniApiEndpoint() . 'ajax/send-mail';
            $response            = $this->httpClient->post($api, [
                'body' => $formData,
            ]);
            $httpResponse        = $response->getStatusCode();
            if ($httpResponse == 200)
            {
                $apiResponse                        = json_decode($response->getBody()->getContents(), true);
                $apiResponseStatus                  = $apiResponse['status'];
                $apiResponseMessage                 = array_get($apiResponse, 'message', $apiResponseMessage);
                $apiResponseArr['response_message'] = $apiResponseMessage;
                if ($apiResponseStatus == 'success')
                {
                    return ['status' => true, 'message' => 'Request has been submitted successfully', 'api_response' => $apiResponseArr];
                }
                else
                {
                    return ['status' => false, 'message' => $apiResponseMessage, 'api_response' => $apiResponseArr];
                }
            }
            else
            {
                $apiResponseArr = ['response_message' => $apiResponseMessage];
                return ['status' => false, 'message' => $apiResponseMessage, 'api_response' => $apiResponseArr];
            }
        }
        //End
        if (!$is_contact)
        {
            return ['status' => false, 'message' => $apiResponseMessage];
        }
        $firstName = array_get($formData, 'firstName', '');
        $to        = array_get($formData, 'to', Config::get('funnel.support_email'));
        $usertFrom = array_get($formData, 'email', '');
        $subject   = array_get($formData, 'subject', 'Contact us');

        //first send the mail to site vendor
        $body = '<div>
                <p style="text-align:center; color:#777;"><h1>' . $subject . '</h1><hr></hr></p>
                <p>Hi, </p>
                <p>A guest user wants to get in touch with us, please find details below.</p>
                <table cellpadding="0" cellspacing="0" style="width:100%">';
        foreach ($formData as $fi => $fv)
        {
            if (
                    $fi == 'grecaptcha_value' || $fi == 'reply_message' ||
                    $fi == 'smtp_setting_id' || $fi == 'is_contact'
            )
            {
                continue;
            }

            $body .= "<tr> <td><b> $fi :</b></td><td>&nbsp;&nbsp;</td><td> $fv</td></tr>";
        }
        $body      .= '
                </table>
                <p>Thanks</p></div>';
        $mailParam = ['to' => $to, 'from' => $usertFrom, 'message' => $body, 'subject' => $subject];
        $mailSent  = $this->sendMail($mailParam);
        //End
        //Send thank yuo notification mail to user
        $body = '<div>
                <p>Hi ' . ucfirst($firstName) . ', </p>
                <p>We\'ve got your request, and will get back to you shortly!</p>
                <p>Thanks</p></div>';
        if (isset($formData['reply_message']) && !empty($formData['reply_message']))
        {
            $body = $formData['reply_message'];
        }
        $mailParam = ['to' => $usertFrom, 'from' => $to, 'message' => $body, 'subject' => $subject];
        $this->sendMail($mailParam);

        return ['status' => $mailSent ? 'success' : false, 'message' => 'Sent'];
    }

    public function trigger($request)
    {
        try
        {
            $requestData = $request->json;
            // 1: OrderConfirmationMail
            if ($requestData['triggerType'] == 'OrderConfirmationMail')
            {
                $responseData = $this->triggerMail($requestData);
            }
            // 2: trackGovMail
            if ($requestData['triggerType'] == 'trackGovMail')
            {
                if (!empty($_SESSION['prospect_shipping_data']))
                {
                    $emailProvided = $_SESSION['prospect_shipping_data']['email'];

                    if (strpos($emailProvided, ".gov") !== false)
                    {
                        $responseData = $this->trackGovMail($requestData['processType']);
                    }
                }
            }
            // 3: triggerTraffic
            if ($requestData['triggerType'] == 'triggerTraffic')
            {
                $responseData = $this->triggerTraffic($requestData['page_url']);
            }

            return $responseData;
        }
        catch (\Exception $e)
        {
            return pre_response_array(null, false, ['response_message' => 'The request couldn\'t be completed! Please try again later'], null, null, $e);
        }
        return pre_response_array(null, false, 'Error!');
    }

    /** Class Private methods starts */
    private function triggerTraffic($pageUrl = null)
    {
        $response = $this->httpClient->post(Helper::uniApiEndpoint() . 'ajax/trigger-traffic', [
            'body' => [
                'funnel_id'       => Helper::getFunnelId(),
                'page_url'        => $pageUrl ? $pageUrl : app()->container()->get('funnelBaseUrl'), // window.location.href
                'ip_address'      => Helper::getIP(),
                'http_user_agent' => app()->container()->get('httpUserAgent')
            ],
        ]);

        $httpResponseCode = $response->getStatusCode();

        if ($httpResponseCode == 200)
        {
            $apiResponse = json_decode($response->getBody()->getContents(), true);
            return pre_response_array(null, ($apiResponse['status'] == 'success') ? true : false, [
                'response_message' => $apiResponse['message'],
            ]);
        }

        return pre_response_array(null, false, ['response_message' => 'The request couldn\'t be completed! Please try again later']);
    }

    private function triggerMail($requestData)
    {
        if (Config::get('events.send_order_confirm_responder_email') != 1 || $requestData['triggerType'] !== 'OrderConfirmationMail')
        {
            throw new \Exception('Autoresponder failed to initiate');
        }

        if ((array_get($_SESSION, 'mail_triggered') == $requestData['queryStrings']['order_id']) || empty($requestData['template']))
        {
            throw new \Exception('Mail already triggered for this order');
        }

        $mailData                      = $this->prepareMailData(1);
        $mailData['smtp_setting_id']   = Config::get('addons.smtp.config_id');
        //Template selection
        $mailData['email_template_id'] = array_get($requestData, 'template', '');
        //Pass tokens to El Minara
        $mailData['token']             = isset($_SESSION['twig_tokens']) ? $_SESSION['twig_tokens'] : [];
        $mailData['currency']          = Config::get('funnel.currency.symbol');

        $response = $this->httpClient->post(Helper::uniApiEndpoint() . 'ajax/send-mail', [
            'body' => $mailData,
        ]);

        if ($response->getStatusCode() == 200)
        {
            $apiResponse                        = json_decode($response->getBody()->getContents(), true);
            $apiResponseStatus                  = $apiResponse['status'];
            $apiResponseMessage                 = $apiResponse['message'];
            $apiResponseArr['response_message'] = $apiResponseMessage;

            if ($apiResponseStatus == 'success')
            {
                $_SESSION['mail_triggered'] = $requestData['queryStrings']['order_id'];
                return ['status' => true, 'api_response' => $apiResponseArr];
            }
            else
            {
                return ['status' => false, 'api_response' => $apiResponseArr];
            }
        }
        else
        {
            throw new \Exception('The request couldn\'t be completed! Please try again later');
        }
    }

    private function trackGovMail($process_type)
    {
        $response = $this->httpClient->post(Helper::uniApiEndpoint() . 'ajax/track-mail', [
            'body' => [
                'funnel_id'       => Helper::getFunnelId(),
                'first_name'      => array_get($_SESSION, 'prospect_shipping_data.firstName', ''),
                'last_name'       => array_get($_SESSION, 'prospect_shipping_data.lastName', ''),
                'email_id'        => array_get($_SESSION, 'prospect_shipping_data.email', ''),
                'visitor_ip'      => Helper::getIp(),
                'visitor_country' => isset($_SERVER['HTTP_CF_IPCOUNTRY']) ? $_SERVER['HTTP_CF_IPCOUNTRY'] : '',
                'process_type'    => $process_type,
            ],
        ]);

        if ($response->getStatusCode() == 200)
        {
            $apiResponse = json_decode($response->getBody()->getContents(), true);
            return pre_response_array(null, ($apiResponse['status'] == 'success') ? true : false, [
                'response_message' => $apiResponse['message'],
            ]);
        }

        return pre_response_array(null, false, ['response_message' => 'The request couldn\'t be completed! Please try again later']);
    }

    private function prepareMailData($mailType)
    {
        $mailData        = [];
        $funnelId        = Config::get('funnel.identifier');
        $crmProductPrice = 0.00;
        $productSummary  = '';

        if (Config::get('offer_details.step1.selectedProduct'))
        {
            $i = 1;
            foreach (Config::get('offer_details.step1.selectedProduct') as $selectedProduct)
            {
                $crmProductPrice += $selectedProduct['crm_product_price'];
                $productSummary  .= sprintf(
                        '%s) Product Name: %s, Price: %s, Quantity: %s<br>',
                        $i, $selectedProduct['product_name'], $selectedProduct['crm_product_price'], $selectedProduct['crm_product_qty']
                );
                $i++;
            }
        }
        $shippingPrice = Config::get('offer_details.step1.shipping_price', 0.00);
        $totalPrice    = $crmProductPrice + $shippingPrice;

        if ($mailType == 1)
        {
            $currentStep = $this->getCurrentStep('thankyou');

            $crmProductPrice      = $crmSplitProductPrice = 0.00;
            $shippingPrice        = $splitShippingPrice   = 0.00;
            $totalPrice           = $splitTotalPrice      = 0.00;

            $crmProductPrice      = Config::get('offer_details.step' . $currentStep . '.selectedProduct.0.crm_product_price', 0.00);
            $shippingPrice        = Config::get('offer_details.step' . $currentStep . '.shipping_price', 0.00);
            $crmSplitProductPrice = Config::get('offer_details.step' . $currentStep . '.splitChargeDetails.selectedProduct.0.crm_product_price', 0.00);
            $splitShippingPrice   = Config::get('offer_details.step' . $currentStep . '.splitChargeDetails.shipping_price', 0.00);

            $crmProductPrice      = number_format($crmProductPrice, 2);
            $shippingPrice        = number_format($shippingPrice, 2);
            $crmSplitProductPrice = number_format($crmSplitProductPrice, 2);
            $splitShippingPrice   = number_format($splitShippingPrice, 2);

            $shippingDetail     = array_get($_SESSION, 'prospect_shipping_data', '');
            $billingDetail      = array_get($_SESSION, 'checkout_billing_data', '');
            $totalPrice         = $crmProductPrice + $shippingPrice;
            $splitTotalPrice    = $crmSplitProductPrice + $splitShippingPrice;
            $countryCode        = array_get($shippingDetail, 'country', '');
            $billingCountryCode = array_get($billingDetail, 'billingCountry', '');
            $shippingStateCode  = array_get($shippingDetail, 'state', '');
            $billingStateCode   = array_get($billingDetail, 'billingState', '');
            $countryName        = Helper::getCountryName($countryCode);
            $billingCountry     = Helper::getCountryName($billingCountryCode);
            $shippingStateName  = Helper::getStateName($shippingStateCode);
            $billingStateName   = Helper::getStateName($billingStateCode);

            $mailData = [
                'funnel_id'                  => $funnelId,
                'mail_type'                  => $mailType,
                'to_mail'                    => array_get($_SESSION, 'prospect_shipping_data.email', ''),
                'orderid'                    => array_get($_SESSION, 'checkout_order_id', ''),
                'orderdatetime'              => date('Y-m-d h:i A'),
                'productname'                => Config::get('offer_details.step1.selectedProduct.0.product_name', ''),
                'productprice'               => (string) Config::get('offer_details.step1.selectedProduct.0.crm_product_price', '0.00'),
                'productqty'                 => Config::get('offer_details.step1.selectedProduct.0.crm_product_qty', '1'),
                'billingaddress'             => array_get($_SESSION, 'checkout_billing_data.billingAddress', array_get($_SESSION, 'prospect_shipping_data.address' . '')),
                'billingaddress2'            => '',
                'billingcity'                => array_get($_SESSION, 'checkout_billing_data.billingCity', array_get($_SESSION, 'prospect_shipping_data.city', '')),
                'billingstate'               => Helper::getStateName(array_get($_SESSION, 'checkout_billing_data.billingState', array_get($_SESSION, 'prospect_shipping_data.state' . ''))),
                'billingzipcode'             => array_get($_SESSION, 'checkout_billing_data.billingZip', array_get($_SESSION, 'prospect_shipping_data.zip' . '')),
                'billingcountry'             => Helper::getCountryName(array_get($_SESSION, 'checkout_billing_data.billingCountry', array_get($_SESSION, 'prospect_shipping_data.country' . ''))),
                'shippingfirstname'          => array_get($_SESSION, 'prospect_shipping_data', ''),
                'shippinglastname'           => array_get($_SESSION, 'prospect_shipping_data.lastName', ''),
                'shippingaddress'            => array_get($_SESSION, 'prospect_shipping_data.address', ''),
                'shippingaddress2'           => '',
                'shippingcity'               => array_get($_SESSION, 'prospect_shipping_data.city', ''),
                'shippingstate'              => Helper::getStateName(array_get($_SESSION, 'prospect_shipping_data.state', '')),
                'shippingzipcode'            => array_get($_SESSION, 'prospect_shipping_data.zip', ''),
                'shippingcountry'            => array_get($_SESSION, 'prospect_shipping_data.country', ''),
                'trackingnumber'             => '',
                'productsummary'             => $productSummary,
                'subtotal'                   => (string) number_format($crmProductPrice, 2),
                'shippingprice'              => (string) number_format($shippingPrice, 2),
                'salestax'                   => '0.00',
                'ordertotal'                 => (string) number_format($totalPrice, 2),
                'funnel_name'                => Config::get('funnel.name'),
                'funnel_url'                 => Config::get('funnel.url'),
                'funnel_support_email'       => Config::get('funnel.funnel_support_email'),
                'funnel_support_phone'       => Config::get('funnel.funnel_support_phone'),
                'funnel_path'                => Config::get('funnel.path'),
                'funnel_short_term'          => Config::get('funnel.short_term'),
                'funnel_extended_version'    => Config::get('funnel.extended_version'),
                'data_collection_disclaimer' => Config::get('funnel.disclaimer'),
                'shipping_insurance_price'   => Config::get('offer_details.step' . $currentStep . '.shipping_insurance_price'),
                'rebill_price'               => Config::get('offer_details.step' . $currentStep . '.rebill_price'),
                'autoship_price'             => Config::get('offer_details.step' . $currentStep . '.autoship_price'),
                'trial_period'               => Config::get('offer_details.step' . $currentStep . '.trial_period'),
                'shipping_period'            => Config::get('offer_details.step' . $currentStep . '.shipping_period'),
                'evaluation_period'          => Config::get('offer_details.step' . $currentStep . '.evaluation_period'),
                'credit_card_descriptor'     => Config::get('offer_details.step' . $currentStep . '.credit_card_descriptor'),
                'domain_name'                => Config::get('domain.name'),
                'copyright_name'             => Config::get('domain.copyright_name'),
                'address_inline'             => Config::get('domain.address_inline'),
                'address_formatted'          => Config::get('funnel.funnel_address_formatted'),
                'address_return'             => Config::get('domain.address_return'),
                'notes'                      => Config::get('domain.notes'),
                'currency'                   => html_entity_decode(Config::get('funnel.currency.symbol')),
                'signer_corp_name'           => Config::get('signer.corp_name'),
                'signer_first_name'          => Config::get('signer.first_name'),
                'signer_last_name'           => Config::get('signer.last_name'),
                'signer_phone'               => Config::get('signer.phone'),
                'signer_email'               => Config::get('signer.email'),
                'owner_corp_name'            => Config::get('signer.corp_name'),
                'owner_first_name'           => Config::get('signer.first_name'),
                'owner_last_name'            => Config::get('signer.last_name'),
                'owner_phone'                => Config::get('signer.phone'),
                'owner_email'                => Config::get('signer.email'),
                'corporation'                => Config::get('signer.corporation'),
                'state_of_incorporation'     => Config::get('signer.state_of_incorporation'),
                'shipping_first_name'        => array_get($shippingDetail, 'firstName', ''),
                'shipping_last_name'         => array_get($shippingDetail, 'lastName', ''),
                'shipping_address'           => array_get($shippingDetail, 'address', ''),
                'shipping_city'              => array_get($shippingDetail, 'city', ''),
                'shipping_country'           => $countryName,
                'shipping_state'             => $shippingStateName,
                'shipping_zip'               => array_get($shippingDetail, 'zip'),
                'shipping_price'             => $shippingPrice,
                'split_shipping_price'       => $splitShippingPrice,
                'billing_first_name'         => array_get($billingDetail, 'billingFirstName'),
                'billing_last_name'          => array_get($billingDetail, 'billingLastName'),
                'billing_address'            => array_get($billingDetail, 'billingAddress'),
                'billing_city'               => array_get($billingDetail, 'billingCity'),
                'billing_country'            => $billingCountry,
                'billing_state'              => $billingStateName,
                'billing_zip'                => array_get($billingDetail, 'billingZip'),
                'insurance_provider_name'    => Config::get('offer_details.step' . $currentStep . '.splitChargeDetails.insurance_provider_name'),
                'insurance_provider_logo'    => Config::get('offer_details.step' . $currentStep . '.splitChargeDetails.insurance_provider_logo'),
                'order_id'                   => array_get($_SESSION, 'checkout_order_id'),
                'phone'                      => array_get($shippingDetail, 'phone'),
                'email'                      => array_get($shippingDetail, 'email'),
                'total_price'                => number_format($totalPrice, 2),
                'split_total_price'          => number_format($splitTotalPrice, 2),
            ];

            $productToken = $this->templateToken->getProductTokenVal();
            $mailData     = array_merge($productToken, $mailData);
        }
        return $mailData;
    }

}
