<?php

namespace Framework\Foundation;

use Framework\Modules\GeoLocation;
class Config
{

    private static $config;

    /**
     * Reset all the config to default
     * @param Array|illuminate\Support\Collection $default
     * 
     * @return void
     */
    public static function resetAll($default = [])
    {
        static::$config = collect($default);
    }

    /**
     * Get an existing Config key if key param passed or return the full config collection
     * @param String $key
     * 
     * @return Mixed
     */
    public static function get($key = null, $default=null)
    {
        if (empty($key))
        {
            return static::$config;
        }

        $value = static::resolve(static::$config, $key);

        return $value ? $value : $default;
    }

    /**
     * Set new config
     * @param String $key
     * @param Mixed $value
     * 
     * @return Mixed
     */
    public static function set($key, $value)
    {
        static::$config->put($key, $value);
        return $value;
    }

    /**
     * Checking that the single purchase limit is on or off
     * @return Boolean
     */
    public static function isSinglePurchaseEnable()
    {
        $singlePurchaseEnable = static::get('addons.scheduled-blocker');
        return (!empty($singlePurchaseEnable)) ? true : false;
    }

    /**
     * Get all frontend public configurations
     * 
     * @return Array
     */
    public static function getAllPublic()
    {
        /// check whether apple and google pay with stripe addon is added with stripe payment provider
        $checkprovider = static::get('payment_provider.api_type');
        $check_app_goo_addon = static::get('payment_provider.apple_google_pay_btn');

        if($checkprovider && $checkprovider == 'stripe'){
            if($check_app_goo_addon)
            {
                
                $apple_google_pay_enabled = 1;
            }
            else{
                $apple_google_pay_enabled = 0;
            }
        }
        else
        {
            $apple_google_pay_enabled = 0;
            
        }

        $config = array(
            'ng_app_id'                 => static::get('ng_app_id', '5edf407dd0e6'),
            'countries'                 => static::get('funnel.countries.available', ''),
            'disabledStateTax'          => static::get('disabled-state-tax', 1),
            'stateTax'                  => static::get('addons.state_tax'),
            'error_messages'            => static::get('error_messages'),
            'test_cards'                => static::get('payment_methods.credit_card.test_cards'),
            'block_bins'                => static::get('addons.bin-manager.bin'),
            'is_block_bins'             => static::get('addons.bin-manager') ? 1 : 0,
            'split_charge'              => static::get('offer_details.split_charge'),
            'enable_upsells'            => static::get('offer_details.enable_upsells'),
            'is_armed_forces_in_list'   => static::get('funnel.countries.is_armed_forces_in_list', 0),
            'is_islands_in_list'        => static::get('funnel.countries.is_islands_in_list', 0),
            'cards'                     => static::get('payment_methods.credit_card.types'),
            'funnel_url'                => static::get('funnel.url'),
            'is_3d_validation'          => (int) static::get('addons.3d_validation.enabled'),
            'api_type'                  => static::get('payment_provider.api_type'),
            'api_type_code'             => static::get('payment_provider.api_type_code'),
            'base_url'                  => Helper::getBaseUrl(),
            'enable_currency_converter' => static::get('addons.currency-converter'),
            'is_paay3d_validation'      => static::get('addons.3d-secure.api_key') ? true : false,
            'paay_api_key'              => static::get('addons.3d-secure.api_key'),
            'currency'                  => static::get('funnel.currency.code'),
            'apple_google_pay_enabled'  => $apple_google_pay_enabled,
            'enable_geoloacation_data' => GeoLocation::enabled(),
        );

        return $config;
    }

    /**
     * Get an item from a Collection using "dot" notation.
     * @param \illuminate\Support\Collection $a
     * @param String $path
     * @param Boolen $sanitize
     * 
     * @return Mixed
     */
    protected static function resolve(\illuminate\Support\Collection $a, $path, $sanitize = false)
    {
        $element = array_get($a, $path);

        return is_string($element) && $sanitize ? filter_var($element, FILTER_SANITIZE_STRING) : $element;
    }

}
