<?php

namespace Framework\Foundation;

use GuzzleHttp\Client;

class Helper
{

    /**
     * Encrypt user password
     * @param  string $password 
     * @return string
     */
    public static function encryptPassword($password)
    {
        if (empty($password))
        {
            return false;
        }

        return md5($password);
    }

    public static function getIP()
    {
        $developmentDomains = array_get(app_configs('development'), 'domains', []);

        if (!empty($_SERVER['HTTP_CF_CONNECTING_IP']))
        {
            $ip = $_SERVER['HTTP_CF_CONNECTING_IP'];
        }
        elseif (!empty($_SERVER['HTTP_CLIENT_IP']))
        {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        }
        elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR']))
        {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        }
        elseif (
                in_array(strtolower($_SERVER['HTTP_HOST']), $developmentDomains) ||
                in_array(strtolower($_SERVER['SERVER_NAME']), $developmentDomains) ||
                $_SERVER['REMOTE_ADDR'] == '::1')
        {
            $ip = rand(1, 255) . '.' . rand(1, 255) . '.' . rand(1, 255) . '.' . rand(1, 255);
        }
        elseif (isset($_SESSION['client_ip']) && !empty($_SESSION['client_ip']))
        {
            $ip = $_SESSION['client_ip'];
        }
        else
        {
            $ip = $_SERVER['REMOTE_ADDR'];
        }

        return $ip;
    }

    public static function getBaseUrl()
    {
        // output: /myproject/index.php
        $currentPath = $_SERVER['PHP_SELF'];

        //get path
        $pathInfo = pathinfo($currentPath);
        $path     = trim($pathInfo['dirname'], '/');

        // output: localhost
        $hostName = trim($_SERVER['HTTP_HOST'], '/');

        return rtrim($hostName . '/' . $path, '/') . "/";
    }

    public static function ordinalSuffix($num)
    {
        $num = $num % 100; // protect against large numbers
        if ($num < 11 || $num > 13)
        {
            switch ($num % 10)
            {
                case 1:return 'st';
                case 2:return 'nd';
                case 3:return 'rd';
            }
        }
        return 'th';
    }

    /**
     * Predict funnel_url
     * @return String
     */
    public static function getValidFunnelUrl()
    {
        $funnel_url     = Config::get('funnel.url');
        $funnel_path    = Config::get('funnel.path');
        $serverName     = $_SERVER['SERVER_NAME'];
        $domainProtocol = array_get($_SERVER, 'DOMAIN_PROTOCOL', '');
        $domainScheme   = array_get($_SERVER, 'REQUEST_SCHEME', 'http');
        $domainProtocol = empty($domainProtocol) ? $domainScheme . ':' : $domainProtocol;

        $matchArray = static::prodDevDomains();

        if (!in_array($serverName, $matchArray) && !empty($serverName) && !empty($domainProtocol))
        {
            $funnel_url = $domainProtocol . '//' . $serverName . $funnel_path;
        }

        return $funnel_url;
    }

    /**
     * Get all production and development domains list
     * @return Array
     */
    public static function prodDevDomains()
    {
        return array_merge(
                array_get(app_configs('development'), 'domains', []),
                array_get(app_configs('production'), 'domains', [])
        );
    }

    /**
     * Create a writable file
     * @param String $fileName
     * @param String $path
     * @param String $defaultContent
     */
    public static function createFileifNotExists($fileName, $path, $defaultContent = '')
    {
        $contentLenght = 0;
        try
        {
            if (!empty($fileName) && !empty($path))
            {
                $targetFile = $path . DS . $fileName;
                if (!file_exists($targetFile))
                {
                    $file          = fopen($targetFile, "w");
                    $contentLenght = fwrite($file, $defaultContent);
                    fclose($file);
                }
            }
        }
        catch (\Exception $ex)
        {
            throw new \Exception($ex->getMessage());
        }

        return $contentLenght;
    }

    /**
     * Delete a writable file
     * @param String $fileName
     * @param String $path
     * 
     * @return bool
     */
    public static function deleteFileifExists($fileName, $path)
    {
        try
        {
            if (!empty($fileName) && !empty($path))
            {
                $targetFile = $path . DS . $fileName;
                if (file_exists($targetFile))
                {
                    return unlink($targetFile);
                }
            }
        }
        catch (\Exception $ex)
        {
            throw new \Exception($ex->getMessage());
        }

        return false;
    }

    /**
     * Return json file data from a specific path
     * @param String $path Json file path
     * @param String $file Json file name
     * 
     * @return \illuminate\Support\Collection
     */
    public static function getJsonFileData($path, $file)
    {
        $data = [];
        try
        {

            $filePath = sprintf('%s%s%s', rtrim($path, '\\'), DS, $file);

            if (file_exists($filePath))
            {
                $data = json_decode(file_get_contents($filePath), true);
            }
        }
        catch (\Exception $e)
        {
            //Handle Exception
        }

        return collect($data);
    }

    /**
     * Fetch Currency info based on IP location
     * @return Mixed
     */
    public static function getCurrencyByIp()
    {
        if (!empty($_SESSION['meta']['currency']))
        {
            return $_SESSION['meta']['currency'];
        }
        $httpClient = new Client();
        $ipAddress  = static::getIP();
        $response   = $httpClient->get(sprintf(
                        'http://www.geoplugin.net/json.gp?ip=%s', $ipAddress
        ));
        $body       = $response->getBody();
        $ipData     = json_decode($body->getContents(), true);

        if (!empty($ipData['geoplugin_currencySymbol_UTF8']))
        {
            $_SESSION['meta']['currency'] = array(
                'symbol' => $ipData['geoplugin_currencySymbol_UTF8'],
                'code'   => $ipData['geoplugin_currencyCode'],
            );
            return $_SESSION['meta']['currency'];
        }

        return false;
    }

    /**
     * Rename replace Array key names
     * 
     * @param Array $old_array
     * @param Array $new_keys
     * 
     * @return Array 
     */
    public static function renameArrayKey($old_array, $new_keys)
    {
        $new_array = [];

        foreach ($new_keys as $newKey => $newValue)
        {
            foreach ($old_array as $oldKey => $oldValue)
            {
                if ($newKey == $oldKey)
                {
                    $new_array[$new_keys[$newKey]] = $oldValue;
                    unset($old_array[$oldKey]);
                    unset($new_array[$oldKey]);
                }
                else
                {
                    $new_array[$oldKey] = $oldValue;
                }
            }
        }

        return $new_array;
    }

    /**
     * Validate a credit card number
     * @param $cardNo A credit card number
     * 
     * @return Mixed
     */
    public static function validateCard($cardNo)
    {

        $cardNumber = [
            'visa'     => "/^4[0-9]{12}(?:[0-9]{3})?$/",
            'master'   => "/^(?:5[1-5][0-9]{2}|222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}$/",
            'mastercard' => "/^(?:5[1-5][0-9]{2}|222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}$/",
            'maestro'  => "/^(5018|5020|5038|5612|5893|6304|6390|6759|676[1-3]|0604)/",
            'amex'     => "/^3[47][0-9]{13}$/",
            'diners'   => "/^3(?:0[0-5]|[68][0-9])[0-9]{11}$/",
            'discover' => "/^6(?:011|5[0-9]{2})[0-9]{12}$/",
            'jcb'      => "/^(?:2131|1800|35\d{3})\d{11}$/",
            'solo'     => "/^(6334|6767)/",
            'laser'    => "/^(6304|670[69]|6771)/"
        ];

        foreach ($cardNumber as $card => $pattern)
        {
            if (preg_match($pattern, str_replace(" ", "", $cardNo)))
            {
                return $card;
            }
        }
        return null;
    }

    /**
     * Get a country name by country code
     * @param string $countryCode
     * 
     * @return Mixed Null|string
     */
    public static function getCountryName($countryCode = null)
    {
        if (empty($countryCode))
            return null;

        $countryList = Helper::getJsonFileData(BASE_PATH . DS . 'frontend' . DS . 'storage', 'country_state.json');

        $country = $countryList->where('FIELD2', $countryCode)->first();

        return !empty($country['FIELD1']) ? $country['FIELD1'] : null;
    }

    /**
     * Get a state name by state code
     * @param string $stateCode
     * 
     * @return Mixed Null|String
     */
    public static function getStateName($stateCode = null)
    {
        if (empty($stateCode))
            return null;

        $stateList = Helper::getJsonFileData(BASE_PATH . DS . 'frontend' . DS . 'storage', 'country_state.json');
        $state     = $stateList->whereIn('FIELD4', [
                    $stateCode,
                    'US-' . $stateCode,
                    'CA-' . $stateCode,
                ])->first();

        return !empty($state['FIELD3']) ? $state['FIELD3'] : null;
    }

    /**
     * Check PDO supproted drivers if available or not
     * @param string $driverName | the pdo driver name (sqlite, mysql ...)
     *  @return bool
     */
    public static function hasPDOdriver($driverName) {

        if(!class_exists('PDO')) return false;

        $getAvailableDrivers = \PDO::getAvailableDrivers();
        if(empty($getAvailableDrivers) || !is_array($getAvailableDrivers)) {
            return false;
        }

        if($driverName == 'sqlite') {
            if(!extension_loaded('pdo_sqlite') || !in_array('sqlite', $getAvailableDrivers)) {
                return false;
            }
            return true;
        }

        if($driverName == 'mysql') {

            if(!extension_loaded('pdo_mysql') || !in_array('sqlite', $getAvailableDrivers)) {
                return false;
            }

            return true;
        }

        return false;
    }

    public static function getDatabseConnection($conn=null) {
    
        $defaultConnections = [
                'mysql_connection' => [
                    'default' => true,
                    'driver' => 'mysql',
                    'host' => 'localhost',
                    'port' => '3306',
                    'database' => 'cms',
                    'username' => 'root',
                    'password' => ''
                ],
                'sqlite_connection' => [
                    'default' => false,
                    'driver' => 'sqlite',
                    'database' => 'uni_app'
                ]
            ];
        
        $connections = array_merge($defaultConnections, Config::get('database.connections', []));

        return $connections[!empty($conn) ? $conn : Config::get('database.default_connection', UNI_APP_DB_CONN)];
    }

    public static function uniApiEndpoint() {
        return rtrim(Config::get('uni_api_endpoint'), "/") . "/";
    }

    public static function numToAlpha($n) {
        $r = '';
        for ($i = 1; $n >= 0 && $i < 10; $i++) {
        $r = chr(0x41 + ($n % pow(26, $i) / pow(26, $i - 1))) . $r;
        $n -= pow(26, $i);
        }
        return $r;
    }

    public static function alphaToNum($a) {
        $r = 0;
        $l = strlen($a);
        for ($i = 0; $i < $l; $i++) {
        $r += pow(26, $i) * (ord($a[$l - $i - 1]) - 0x40);
        }
        return $r - 1;
    }

    public static function getFunnelId(){
        $funnelId = static::alphaToNum(Config::get('funnel.identifier'));
        return !empty($funnelId) && $funnelId>0 ? $funnelId : NULL;
    }

}
