<?php

namespace Framework\Middleware;

use Closure;
use Framework\Foundation\Config;

class InitShippingProfile implements \Framework\Contracts\Middleware {

    public function handle($request, Closure $next)
    {
        $rawRequest = $request->getContent();
        $formData = json_decode($rawRequest, true);

        $sid = array_get($formData, 'shippingId', array_get($formData, 'upsellData.shippingId', null));

        if(!$sid) $sid = array_get($_SESSION, 'external_data.shipping_id', null);

        $this->initShippingProfile($sid);
        
        return $next($request);
    }

    private function initShippingProfile($sid)
    {
        if($sid) {
            $shippingProfile = collect(Config::get('shipping_profile'))->where('shipping_profile_crm_id', $sid)->first();
            if(empty($shippingProfile))
            {
                $shippingProfile = collect(Config::get('shipping_profile'))->where('id', $sid)->first();
            }
        } else {
            $shippingProfile = collect(Config::get('shipping_profile'))->where('default_shipping', 1)->first();
        }

        if(!$shippingProfile) {

            $shippingProfile = $this->generateDefaultShippingProfile();
        }

        if(!$shippingProfile) throw new \Exception("Invalid shipping profile", AJAX_EX_CODE);
        Config::set('selected_shipping_profile', collect($shippingProfile));
        // $shipping_id = $this->shippingProfile->get('shipping_profile_crm_id');
        // $shipping_price = $this->shippingProfile->get('shipping_profile_price');
    }


    /**
     * Generate default shipping profile for Non shipping required CRM
     */
    private function generateDefaultShippingProfile() {

        $crmType = Config::get('payment_provider.api_type');

        if(in_array($crmType, ['demoCRM', 'response', 'twentyNineNext', 'sixcrm', 'square', 'konnektive', 'sublytics', 'stripe','vrio'])) {

            return [
                "shipping_profile_name" => "free_shipping",
                "shipping_profile_price" => "0.00",
                "default_shipping" => 1,
                "shipping_profile_description" => "Free Shipping",
                "shipping_profile_crm_id" => null,
                "shipping_profile_provider" => null,
                "custom_fields" => []
            ];
        }

        return null;
    }
 
}


