<?php

namespace Framework\Models;

use Framework\Foundation\Config;

class CountryStateCity extends Model {

    public function __construct()
    {
        $this->setupDatabaseConfig();
        Parent::__construct();
    }

    private function setupDatabaseConfig() {
        $customConnections = collect(Config::get('database.connections', []))->merge([
            'sqlite_connection_2' => [
                'default' => false,
                'driver' => 'sqlite',
                'database' => 'geolocation_data'
                ]
        ])->toArray();

        $customModels = collect(Config::get('database.models', []))->merge([
            $this->getModelName() => [
                'database_connections' => 'sqlite_connection_2'
                ]
        ])->toArray();
        Config::set('database', ['connections' => $customConnections, 'models' => $customModels]);
    }

    protected function migrations() {

        if($this->tableSchema) return $this->tableSchema;

        // As we are not creating this table on run time we will download the DB with the
        // table and all the required data from our storage, so that if required we can 
        // locally update the database on framework update
        $this->updateLocalDb();

        $query = sprintf( "CREATE TABLE IF NOT EXISTS %s (", $this->table);
        $query .= "`id` INTEGER PRIMARY KEY /*!40101 AUTO_INCREMENT */, ";
        $query .= "`country_code` varchar(2) DEFAULT NULL, ";
        $query .= "`state_code` varchar(5) DEFAULT NULL, ";
        $query .= "`city_name` varchar(100) DEFAULT NULL) ";

        return $query;
    }

    private function updateLocalDb() {
        $result = file_put_contents(
            $this->SQLiteDbPath() .DS.'geolocation_data.sqlite', 
            file_get_contents('https://storage.googleapis.com/hosted-cms/assets/addons/geolocation_data.sqlite')
        );

        if(!$result) throw new \Exception('Something went wrong!');
    }
}