<?php

namespace Framework\Modules\Crm;

use Framework\Modules\Crm\Contracts\Crm;

class Adapter
{

    private $crmProvider;

    public function __construct(Crm $crm)
    {
        $this->crmProvider = $crm;
    }

    /**
     * Patch for method calling which is not defined in every crm provider class.
     */
    public function __call($name, $arguments)
    {

        if (!method_exists($this->crmProvider, $name))
        {
            throw new \Exception(sprintf("The %s CRM provider does not support the %s method!", get_class($this->crmProvider), $name));
        }

        return call_user_func_array(array($this->crmProvider, $name), $arguments);
    }

    public function validateCoupon($formData)
    {
        $response = $this->crmProvider->constructEnv()->validateCoupon($formData);

        return $response;
    }

    public function newProspect($formData)
    {
        $response = $this->crmProvider->constructEnv()->newProspect($formData);

        return $response;
    }

    /**
     * @param array $formData
     */
    public function newOrderWithProspect($formData)
    {
        $response = $this->crmProvider->constructEnv()->newOrderWithProspect($formData);
        $step = 1;
        $request = $formData;
        \Framework\Events\NewOrderPlaced::dispatch(compact('response', 'request', 'step'));

        return $response;
    }

    /**
     * @param array $formData
     * @param array $configData
     * @param string $type
     */
    public function newOrder($formData, $configData, $type)
    {
        $response = $this->crmProvider->constructEnv()->newOrder($formData, $configData, $type);

        return $response;
    }

    /**
     * @param array $queryStrings
     * @param array $orderIds
     */
    public function orderView($queryStrings, $orderIds)
    {
        $response = $this->crmProvider->constructEnv()->orderView($queryStrings, $orderIds);

        return $response;
    }

    /**
     * @param array $formData
     */
    public function singleOrderView($formData)
    {
        $response = $this->crmProvider->constructEnv()->singleOrderView($formData);

        return $response;
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function hasOwnedThisOrder($responseData, $formData)
    {
        $response = $this->crmProvider->constructEnv()->hasOwnedThisOrder($responseData, $formData);

        return $response;
    }

    /**
     * Check customer email with order email
     * @param array $orderViewResponse
     * @param string $email
     * 
     * @return bool
     */
    public function orderBelongsToTheEmail($email, $orderViewResponse)
    {
        $response = $this->crmProvider->constructEnv()->hasOwnedThisOrder($orderViewResponse, compact('email'));

        return $response;
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function prepareDataForCancel($responseData, $formData)
    {
        $response = $this->crmProvider->constructEnv()->prepareDataForCancel($responseData, $formData);

        return $response;
    }

    /**
     * @param array $formData
     */
    public function cancelOrder($formData)
    {
        $response = $this->crmProvider->constructEnv()->cancelOrder($formData);

        return $response;
    }

    /**
     * @param array $response
     */
    public function isDecline($response)
    {
        $response = $this->crmProvider->constructEnv()->isDecline($response);

        return $response;
    }

    public function placeUpsellOrder($request, $step)
    {

        $response = $this->crmProvider->constructEnv()->placeUpsellOrder($request, $step);

        \Framework\Events\UpsellOrderPlaced::dispatch(compact('response', 'request', 'step'));

        return $response;
    }

    public function placeNewOrder($request, $step)
    {

        $response = $this->crmProvider->constructEnv()->placeNewOrder($request, $step);

        \Framework\Events\NewOrderPlaced::dispatch(compact('response', 'request', 'step'));

        return $response;
    }

}
