<?php

namespace Framework\Modules\Crm\Middleware;

use Closure;
use Framework\Foundation\Config;

class FilterRequestObject implements \Framework\Contracts\Middleware
{

    public function handle($request, Closure $next)
    {
        $formData = $request->json;

        if (!empty($formData['externalResponse']) && $formData['externalResponse'] == 1)
        {
            return pre_response_array(null, false, array(
                'response_message' => array_get($formData, 'externalMessage', 'Unable to process!')
            ));
        }

        if (!empty($formData['campaignId'])) $_SESSION['external_data'] = ['campaign_id' => $formData['campaignId']];
        if (!empty($formData['shippingId'])) $_SESSION['external_data']['shipping_id'] = $formData['shippingId'];
        
        $productData = $this->prepareProductData($formData);

        if (!empty($productData)) $_SESSION['external_data']['product_data'] = $productData;
        if (isset($formData['campaignId'])) unset($formData['campaignId']);
        if (isset($formData['shippingId'])) unset($formData['shippingId']);
        if (isset($formData['productId'])) unset($formData['productId']);
        if (isset($formData['productPrice'])) unset($formData['productPrice']);
        if (isset($formData['productQty'])) unset($formData['productQty']);

        $this->prepareCardData($formData);
        $request->json = $formData;
        return $next($request);
    }

    private function prepareProductData($formData)
    {

        if (empty($formData['productId']) && empty($formData['productPrice']) && empty($formData['productQty']))
        {
            return [];
        }

        $selectedProduct = Config::get('offer_details.step1.selectedProduct');
        $productDetails  = [];

        foreach ($selectedProduct as $pi => $pv)
        {
            $productDetails[$pv['crm_product_id']]      = $pv;
            $productDetails['elm_' . $pv['product_id']] = $pv;
        }

        $product_ids  = explode(",", $formData['productId']);
        $product_qtys = !empty($formData['productQty']) ? explode(",", $formData['productQty']) : [];
        $campaign_ids = !empty($formData['campaignId']) ? explode(",", $formData['campaignId']) : [];
        $product_prices = !empty($formData['productPrice']) ? explode(",", $formData['productPrice']) : [];

        foreach ($product_ids as $pi => $pv)
        {
            $pv = trim($pv);
            if (empty($pv) || !isset($productDetails[$pv]))
                continue;

            $p_cpid            = $productDetails[$pv]['crm_product_id'];
            $product_id        = $productDetails[$pv]['product_id'];
            $p_qty             = !empty($product_qtys[$pi]) ? trim($product_qtys[$pi]) : $productDetails[$pv]['crm_product_qty'];
            $p_qty             = !empty($product_qtys[$pi]) ? trim($product_qtys[$pi]) : $productDetails[$pv]['crm_product_qty'];
            
            $p_cid             = !empty($campaign_ids[$pi]) ? trim($campaign_ids[$pi]) : '';
            $product_group_key = array_get($productDetails, sprintf("%s.product_group_key", $pv), '');

            if(!empty($product_prices[$pi])) {
                $p_price = trim($product_prices[$pi]);
            } else {
                $p_price = !empty($productDetails[$pv]['crm_product_price']) ? $productDetails[$pv]['crm_product_price'] : '';
            }

            $p_sid   = array_get($productDetails[$pv], 'extra_fields.crm_product_shipping_id', '');
            $s_price = array_get($productDetails[$pv], 'extra_fields.crm_product_shipping_price', '');
            $p_sku = !empty($productDetails[$pv]['product_sku']) ? $productDetails[$pv]['product_sku'] : '';

            $productData[] = [
                'crm_product_id'             => $p_cpid,
                'product_id'                 => $product_id,
                'crm_product_price'          => $p_price,
                'crm_product_qty'            => $p_qty,
                'crm_campaign_id'            => $p_cid,
                'crm_shipping_id'            => $p_sid,
                'product_group_key'          => $product_group_key,
                'crm_product_shipping_price' => $s_price,
                'offer_id'                   => array_get($productDetails[$pv], 'extra_fields.offer_id'),
                'billing_model_id'           => array_get($productDetails[$pv], 'extra_fields.billing_model_id'),
                'use_as_trial_product'       => array_get($productDetails[$pv], 'extra_fields.use_as_trial_product'),
                'trial_product_id'           => array_get($productDetails[$pv], 'extra_fields.trial_product_id'),
                'product_sku'                => $p_sku
            ];
        }

        return $productData;
    }

    private function prepareCardData(&$formData) {
        // $crmType = Config::get('payment_provider.api_type');

        if (!empty($formData['cc_type'])) {
            if ($formData['cc_type'] === 'mastercard') $formData['cc_type'] = 'master';
            if ($formData['cc_type'] === 'visa_electron') $formData['cc_type'] = 'visa';
        }
        if(!empty($formData['paypal_payment'])){
            $formData['cc_type'] = 'paypal';
            unset($formData['cc_number']);
            unset($formData['expmonth']);
            unset($formData['expyear']);
            unset($formData['cvv']);
        }
    }
}
