<?php
namespace Framework\Modules\Crm\Middleware;

use Closure;
use Exception;
use Framework\Foundation\Config;
use Framework\Foundation\Helper;
use Framework\Modules\IpQualityScore;

class QualityScore implements \Framework\Contracts\Middleware
{
    private $isFraud = false;
    private $isEmailFraud = false;
    private $ipQualityConfig;
    private $ipqualityscore;

    public function handle($request, Closure $next)
    {
        try{
            $this->ipQualityConfig = Config::get('addons.ipquality-score');
            if(!$this->ipQualityConfig) return $next($request);

            $this->ipqualityscore = new IpQualityScore(
                Config::get('addons.ipquality-score.end_point'), Config::get('addons.ipquality-score.api_key')
            );
            
            if($this->isIPFailed()) $this->isFraud = true;

            if($this->ipQualityConfig['enable_email_verify']) {
                if($this->isEmailFailed()) $this->isEmailFraud = true;
            }
            
            if($this->isFraud || $this->isEmailFraud) {
                return array_merge(pre_response_array(null, false, [
                    'response_message' => $this->ipQualityConfig['err_msg']
                ]), ['isActiveCaptcha' => $this->ipQualityConfig['show_google_capcha']]);
            }

        }
        catch(\Exception $e){
            throw new Exception($e->getMessage() . " in file: ". $e->getFile() . " on line: " . $e->getLine());
        }
        
        return $next($request);
    }

    private function isIPFailed()
    {

        $response = $this->ipqualityscore->ipFraudScore(Helper::getIP());

        if(!empty($response['fraud_score']) && ($response['fraud_score'] > $this->ipQualityConfig['fraud_score'])) {
            return true;
        }
        
        if($response['is_crawler'] || $response['recent_abuse'] || $response['bot_status']) {
            return true;
        }

        if($this->ipQualityConfig['allowed_country'] && !empty($response['country_code'])) {
            $countryArray = explode(',', $this->ipQualityConfig['allowed_country']);
            if(!in_array($response['country_code'], $countryArray)) return true;
        }

        if($this->ipQualityConfig['vpn']) {
            if($response['vpn'] || $response['active_vpn']) return true;
        }
        
        if($this->ipQualityConfig['tor']) {
            if($response['tor'] || $response['active_tor']) return true;
        }

        return false;
    }

    private function isEmailFailed()
    {
        $emailAddress = array_get($_SESSION,'prospect_shipping_data.email');

        if(empty($emailAddress)) return false;

        $response = $this->ipqualityscore->emailFraudScore($emailAddress);

        if(!empty($response['fraud_score']) && ($response['fraud_score'] > $this->ipQualityConfig['fraud_score'])) {
            return true;
        }

        return false;
    }

}
