<?php

namespace Framework\Modules\Crm\Middleware\Upsell;

use Closure;
use Framework\Foundation\Config;

class FilterRequestObject implements \Framework\Contracts\Middleware {

    public function handle($request, Closure $next)
    {
        $formData = $request->json;

        $upsellData = $formData['upsellData'];

        if (!empty($upsellData['externalResponse']) && $upsellData['externalResponse'] == 1)
        {
            $message = 'Unable to process!';

            if (!empty($upsellData['externalMessage']))
            {
                $message = $upsellData['externalMessage'];
            }
    
            return pre_response_array(null, false, array('response_message' => $message));
        }


        if (!empty($upsellData['campaignId']))
        {
            $_SESSION['external_data'] = [
                'campaign_id' => $upsellData['campaignId']
            ];
        }

        if (!empty($upsellData['shippingId']))
        {
            $_SESSION['external_data']['shipping_id'] = $upsellData['shippingId'];
        }

        $productData = $this->prepareProductData($upsellData);

        if (!empty($productData))
        {
            $_SESSION['external_data']['product_data'] = $productData;
            // //override shippingId if radio type cart based checkout, so shippingId is taken from that product's shippingId field
            // if (isset($productData[0]) && !empty($productData[0]['crm_shipping_id']))
            // {
            //     $_SESSION['external_data']['shipping_id'] = $productData[0]['crm_shipping_id'];
            // }
        }

        if (!empty($upsellData['cc_type']) && $upsellData['cc_type'] === 'visa_electron')
        {
            $upsellData['cc_type'] = 'visa';
        }

        if (isset($upsellData['campaignId']))
        {
            unset($upsellData['campaignId']);
        }
        if (isset($upsellData['shippingId']))
        {
            unset($upsellData['shippingId']);
        }
        if (isset($upsellData['productId']))
        {
            unset($upsellData['productId']);
        }
        if (isset($upsellData['productPrice']))
        {
            unset($upsellData['productPrice']);
        }
        if (isset($upsellData['productQty']))
        {
            unset($upsellData['productQty']);
        }
        
        $formData['upsellData'] = $upsellData;
        $request->json = $formData;
        return $next($request);
    }

    private function prepareProductData($formData) {

        if (empty($formData['productId']) && empty($formData['productPrice']) && empty($formData['productQty']))
        {
            return [];
        }

        $selectedProduct = Config::get('offer_details.step1.selectedProduct');
        $productDetails  = [];
        
        foreach ($selectedProduct as $pi => $pv)
        {
            $productDetails[$pv['crm_product_id']]      = $pv;
            $productDetails['elm_' . $pv['product_id']] = $pv;
        }

        $product_ids  = explode(",", $formData['productId']);
        $product_qtys = isset($formData['productQty']) && !empty($formData['productQty']) ? explode(",", $formData['productQty']) : [];
        $campaign_ids = isset($formData['campaignId']) && !empty($formData['campaignId']) ? explode(",", $formData['campaignId']) : [];

        foreach ($product_ids as $pi => $pv)
        {
            $pv = trim($pv);
            if (empty($pv) || !isset($productDetails[$pv])) continue;

            $p_cpid            = $productDetails[$pv]['crm_product_id'];
            $product_id        = $productDetails[$pv]['product_id'];
            $p_qty             = isset($product_qtys[$pi]) && !empty($product_qtys[$pi]) ? trim($product_qtys[$pi]) : $productDetails[$pv]['crm_product_qty'];
            $p_cid             = isset($campaign_ids[$pi]) && !empty($campaign_ids[$pi]) ? trim($campaign_ids[$pi]) : '';
            $p_sid             = isset($productDetails[$pv]['crm_product_shipping_id']) && !empty($productDetails[$pv]['crm_product_shipping_id']) ? $productDetails[$pv]['crm_product_shipping_id'] : '';
            $product_group_key = isset($productDetails[$pv]['product_group_key']) && !empty($productDetails[$pv]['product_group_key']) ? $productDetails[$pv]['product_group_key'] : '';

            $p_price = $productDetails[$pv]['crm_product_price'];
            $s_price = $productDetails[$pv]['crm_product_shipping_price'];

            $productData[] = [
                'crm_product_id'             => $p_cpid,
                'product_id'                 => $product_id,
                'crm_product_price'          => $p_price,
                'crm_product_qty'            => $p_qty,
                'crm_campaign_id'            => $p_cid,
                'crm_shipping_id'            => $p_sid,
                'product_group_key'          => $product_group_key,
                'crm_product_shipping_price' => $s_price,
                'offer_id'                   => array_get($productDetails[$pv], 'extra_fields.offer_id'),
                'billing_model_id'           => array_get($productDetails[$pv], 'extra_fields.billing_model_id'),
                'use_as_trial_product'       => array_get($productDetails[$pv], 'extra_fields.use_as_trial_product'),
                'trial_product_id'           => array_get($productDetails[$pv], 'extra_fields.trial_product_id'),
              ];
        }

        return $productData;
    }
}