<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Foundation\Helper;
use GuzzleHttp\Client;

abstract class Crm
{

    public function __construct() {}

    public function constructEnv()
    {
        $this->api_base_url = Config::get('payment_provider.api_end_point');
        $this->username     = Config::get('payment_provider.api_user_name');
        $this->password     = Config::get('payment_provider.api_password');

        $this->campaignId       = array_get($_SESSION, 'external_data.campaign_id', Config::get('offer_details.step1.campaign_id'));

        $this->shippingId = Config::get('selected_shipping_profile')->get('shipping_profile_crm_id');
        $this->shippingProfile = Config::get('selected_shipping_profile');
        $this->selectedProducts = array_get($_SESSION, 'external_data.product_data', $this->mapSelectedProducts(Config::get('offer_details.step1.selectedProduct')));

        $this->ipAddress = Helper::getIP();
        $this->notes     = Config::get('domain.notes');

        $this->httpClient = new Client();
        $this->httpClient->setDefaultOption('verify', false);

        $this->forceGatewayId   = Config::get('offer_details.step1.crm_force_gateway_id');
        $this->skip_website_url = (!empty(Config::get('addons.order-notes.skip_website_url')) && in_array('order', Config::get('addons.order-notes.skip_website_url'))) ? true : false;
        return $this;
    }

    protected function mapSelectedProducts($selectedProduct) {

        foreach($selectedProduct as &$product) {
            if(!array_get($product, 'extra_fields.offer_id')) continue;
            $product['offer_id'] = array_get($product, 'extra_fields.offer_id');
            $product['billing_model_id'] = array_get($product, 'extra_fields.billing_model_id');
            $product['use_as_trial_product'] = array_get($product, 'extra_fields.use_as_trial_product');
            $product['trial_product_id'] = array_get($product, 'extra_fields.trial_product_id');
        }

        return $selectedProduct;
    }

    /**
     * Depricated
     * Need to check for legacy support
     */
    public function placeSplitOrder($request, $step)
    {

        $splitChargeDetails = Config::get(sprintf('offer_details.step%s.splitChargeDetails', $step));

        $configData = [
            'selectedProducts' =>  $this->mapSelectedProducts(array_get($splitChargeDetails, 'selectedProduct')),
            'campaignId'       => array_get($splitChargeDetails, 'campaign_id'),
            'shippingId'       => array_get($splitChargeDetails, 'shipping_id'),
            'forceGatewayId'   => array_get($splitChargeDetails, 'crm_force_gateway_id'),
            'prospect_id'      => array_get($request, 'prospect_id', ''),
            'customer_id'      => array_get($request, 'customer_id', ''),
        ];

        return $this->newOrder($request, $configData, 'SplitOrder');
    }

    public function placeUpsellOrder($request, $step)
    {

        $offerConfig = Config::get('offer_details.step' . $step);

        $configData = [
            'selectedProducts' => $this->mapSelectedProducts( array_get($offerConfig, 'selectedProduct')),
            'campaignId'       => array_get($_SESSION, 'external_data.campaign_id', array_get($offerConfig, 'campaign_id')),
            'shippingId'       => $this->shippingProfile->get('shipping_profile_crm_id'),
            'forceGatewayId'   => array_get($offerConfig, 'crm_force_gateway_id'),
            'prospect_id'      => array_get($request, 'prospect_id', ''),
            'customer_id'      => array_get($request, 'customer_id', ''),
        ];

        return $this->newOrder($request, $configData, 'UpsellOrder');
    }

    public function placeNewOrder($request, $step)
    {

        $offerConfig      = Config::get('offer_details.step' . $step);
        $selectedProducts = array_get($_SESSION, 'external_data.product_data', $this->mapSelectedProducts(array_get($offerConfig, 'selectedProduct')));

        $configData = [
            'selectedProducts' => $selectedProducts,
            'campaignId'       => array_get($_SESSION, 'external_data.campaign_id', array_get($offerConfig, 'campaign_id')),
            'shippingId'       => $this->shippingProfile->get('shipping_profile_crm_id'),
            'forceGatewayId'   => array_get($offerConfig, 'crm_force_gateway_id'),
            'prospect_id'      => array_get($request, 'prospect_id', ''),
            'customer_id'      => array_get($request, 'customer_id', ''),
        ];

        return $this->newOrder($request, $configData, 'NewOrder');
    }

}
