<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;
use Framework\Foundation\Helper;
use GuzzleHttp\Client;

class Konnektive extends Crm implements CrmContracts
{
    protected $api_base_url;
    protected $loginId;
    protected $password;
    protected $campaignId;
    protected $shippingId;
    protected $forceGatewayId;
    protected $selectedProducts;
    protected $ipAddress;
    protected $httpClient;
    protected static $methods  = [
        'NewProspect'          => 'leads/import',
        'NewOrderWithProspect' => 'order/import',
        'UpsellOrder'          => 'upsale/import',
        'NewOrder'             => 'order/import',
        'OrderView'            => 'purchase/query',
        'ConfirmOrder'         => 'order/confirm',
        'OrderDetails'         => 'order/query',
        'CancelOrder'          => 'order/cancel',
        'AddCustomerNote'      => 'customer/addnote'
    ];
    protected static $formKeys = [
        'NewProspect'          => [
            'address' => 'address1',
            'zip'     => 'postalCode',
            'phone'   => 'phoneNumber',
            'email'   => 'emailAddress'
        ],
        'NewOrderWithProspect' => [
            'billingSameAsShipping' => 'billShipSame',
            'firstName'             => 'shipFirstName',
            'lastName'              => 'shipLastName',
            'address'               => 'shipAddress1',
            'city'                  => 'shipCity',
            'country'               => 'shipCountry',
            'state'                 => 'shipState',
            'zip'                   => 'shipPostalCode',
            'phone'                 => 'phoneNumber',
            'email'                 => 'emailAddress',
            'billingFirstName'      => 'firstName',
            'billingLastName'       => 'lastName',
            'billingAddress'        => 'address1',
            'billingCity'           => 'city',
            'billingCountry'        => 'country',
            'billingState'          => 'state',
            'billingZip'            => 'postalCode',
            'cc_number'             => 'cardNumber',
            'expmonth'              => 'cardMonth',
            'expyear'               => 'cardYear',
            'cvv'                   => 'cardSecurityCode',
            'promo_code'            => 'couponCode',
        ],
        'NewOrder'             => [
            'billingSameAsShipping' => 'billShipSame',
            'firstName'             => 'shipFirstName',
            'lastName'              => 'shipLastName',
            'address'               => 'shipAddress1',
            'city'                  => 'shipCity',
            'country'               => 'shipCountry',
            'state'                 => 'shipState',
            'zip'                   => 'shipPostalCode',
            'phone'                 => 'phoneNumber',
            'email'                 => 'emailAddress',
            'cc_number'             => 'cardNumber',
            'expmonth'              => 'cardMonth',
            'expyear'               => 'cardYear',
            'cvv'                   => 'cardSecurityCode',
            'promo_code'            => 'couponCode',
        ],
    ];

    public function __construct()
    {
        parent::__construct();
        $this->loginId                  = Config::get('payment_provider.api_user_name');
        $this->salesUrl                 = Config::get('sales_url', Config::get('funnel.url'));
        $this->skip_website_url_partial = (!empty(Config::get('addons.order-notes.skip_website_url')) && in_array('partial', Config::get('addons.order-notes.skip_website_url'))) ? true : false;
    }

    protected function prepareEndpointApi($api_base_url, $method)
    {
        $methodsArr  = self::$methods;
        $endpointUrl = $api_base_url . sprintf('/%s/?', array_key_exists($method, $methodsArr) ? $methodsArr[$method] : '');
        return $endpointUrl;
    }

    protected function httpRequest($api, $new_form_params)
    {
        // Using Deconstructor to solve cURL error 2 , we need to do R&D for this
        $this->httpClient = null;
        $this->httpClient = new Client();
        $this->httpClient->setDefaultOption('verify', false);

        $form_params = [
            'loginId'  => $this->loginId,
            'password' => $this->password,
        ];

        $form_params = array_merge_recursive($form_params, $new_form_params);
        $form_params = array_filter($form_params);

        if (!empty($form_params['country']) && !empty($form_params['state']))
        {
            $form_params['state'] = str_replace(
                    $form_params['country'] . '-', '', $form_params['state']
            );
        }

        if (!empty($form_params['shipCountry']) && !empty($form_params['shipState']))
        {
            $form_params['shipState'] = str_replace(
                    $form_params['shipCountry'] . '-', '', $form_params['shipState']
            );
        }

        $response = $this->httpClient->post($api, [
            'body' => $form_params,
        ]);

        return $response;
    }

    protected function callApi($method, $form_post_params)
    {
        $api            = $this->prepareEndpointApi($this->api_base_url, $method);
        $apiResponseArr = [];

        if (Config::get('addons.3d_validation.enabled') && in_array($method, array('NewOrderWithProspect', 'NewOrder', 'UpsellOrder')))
        {
            $gatewayResponseUrl       = sprintf('%s/konnektive-3ds-response', Helper::getValidFunnelUrl());
            $_SESSION['gateway_meta'] = array(
                'current_url' => $form_post_params['current_url'],
                'next_url'    => $form_post_params['next_url'],
            );

            $form_post_params['redirectsTo']      = $gatewayResponseUrl;
            $form_post_params['errorRedirectsTo'] = $gatewayResponseUrl;
        }

        try
        {
            $response = $this->httpRequest($api, $form_post_params);

            $httpResponse = $response->getStatusCode();
            if ($httpResponse == 200)
            {
                $apiResponse = json_decode($response->getBody()->getContents(), true);

                if (
                        !empty($apiResponse['result']) &&
                        $apiResponse['result'] === 'MERC_REDIRECT' &&
                        !empty($apiResponse['message']['script'])
                )
                {
                    $htmlContent = sprintf(
                            '<body><script>%s</script></body>', $apiResponse['message']['script']
                    );
                    header('Content-Type: application/json');
                    echo json_encode(array(
                        'status'    => 'success',
                        'is_html'   => true, 'html_data' => $htmlContent,
                    ));
                    exit;
                }

                $apiResponseStatus  = $apiResponse['result'];
                $apiResponseMessage = $apiResponse['message'];

                if ($apiResponseStatus == 'SUCCESS')
                {
                    $apiResponseArr = $apiResponseMessage;

                    if ($method == 'NewProspect' || $method == 'NewOrder')
                    {
                        $apiResponseArr['prospectId'] = rand(1111, 9999);
                    }

                    if (!empty($apiResponseMessage['orderId']) &&
                            $apiResponseMessage['orderStatus'] === 'COMPLETE')
                    {
                        $orders = array();
                        if (!empty($_SESSION['__orders__']))
                        {
                            $orders = $_SESSION['__orders__'];
                        }
                        $_SESSION['__orders__'] = array_merge(array(
                            $apiResponseMessage['orderId'] => $apiResponseMessage['items'],
                                ), $orders);

                        if (!empty($form_post_params['konnektiveConfirmOrder']))
                        {
                            $this->performConfirmOrder($apiResponseMessage['orderId']);
                        }
                    }

                    return ['status' => true, 'api_response' => $apiResponseArr];
                }
                else
                {
                    $apiResponseArr['response_message'] = $apiResponseMessage;
                    $re                                 = '/(Transaction Declined:)/m';
                    if (preg_match_all($re, $apiResponseMessage, $match))
                    {
                        return ['status' => true, 'api_response' => $apiResponseArr['response_message'], 'isOrderDecline' => true];
                    }
                    return ['status' => false, 'api_response' => $apiResponseArr];
                }
            }
            else
            {
                $apiResponseArr = ['response_message' => 'The request couldn\'t be completed! Please try again later'];
                return ['status' => false, 'api_response' => $apiResponseArr];
            }
        }
        catch (\Exception $ex)
        {
            $apiResponseArr = ['response_message' => 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }

    public function newProspect($formData)
    {
        $method     = 'NewProspect';
        $configData = [];

        $formData['billShipSame'] = true;

        $configData['campaignId'] = $this->campaignId;
        $configData['ipAddress']  = $this->ipAddress;
        $crmData                  = array_merge_recursive($formData, $configData);
        $crmData                  = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
        $crmData                  = array_filter($crmData);

        $singlePurchaseEnable = Config::isSinglePurchaseEnable();
        if ($singlePurchaseEnable)
        {
            $alreadyPurchased = $this->checkAlreadyPurchased($crmData);
            if ($alreadyPurchased)
            {
                $message = Config::get('addons.scheduled-blocker.display_message');
                return ['status' => false, 'api_response' => ['response_message' => $message]];
            }
        }

        $response = $this->callApi($method, $crmData);
        if ($this->skip_website_url_partial)
        {
            $response['customer_add_note_partial'] = $this->addCustomerNote($response);
        }
        else
        {
            $current_url                           = !empty($crmData['current_url']) ? $crmData['current_url'] : ((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https://" : "http://") . $_SERVER["SERVER_NAME"] . $_SERVER["REQUEST_URI"]);
            $response['customer_add_note_partial'] = $this->addCustomerNote($response, $current_url);
        }

        return $response;
    }

    public function newOrderWithProspect($checkoutData)
    {
        $method     = 'NewOrderWithProspect';
        $configData = [];

        $i = 1;
        foreach ($this->selectedProducts as $selectedProduct)
        {
            $checkoutData['product' . $i . '_id']        = $selectedProduct['crm_product_id'];
            $checkoutData['product' . $i . '_qty']       = $selectedProduct['crm_product_qty'];
            $checkoutData['product' . $i . '_price']     = $selectedProduct['crm_product_price'];
            $checkoutData['product' . $i . '_shipPrice'] = $selectedProduct['crm_product_shipping_price'];

            $i++;
        }

        $checkoutData['expyear']   = 20 . $checkoutData['expyear'];
        $checkoutData['paySource'] = !empty($checkoutData['paySource']) ? $checkoutData['paySource'] : 'CREDITCARD';

        $configData['campaignId']    = $this->campaignId;
        $configData['shipProfileId'] = $this->shippingProfile->get('shipping_profile_crm_id');
        $configData['salesUrl']      = $this->salesUrl;
        if ($this->forceGatewayId > 0)
        {
            $configData['forceMerchantId'] = $this->forceGatewayId;
        }

        unset($checkoutData['prospectId']);
        unset($checkoutData['cc_type']);

        $prospectFormData = $_SESSION['prospect_shipping_data'];
        $crmData          = array_merge_recursive($prospectFormData, $checkoutData, $configData);
        $crmData          = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
        $crmData          = array_filter($crmData);

        $crmData['billShipSame'] = ($crmData['billShipSame'] == 'yes') ? 1 : 0;

        $singlePurchaseEnable = Config::isSinglePurchaseEnable();
        if ($singlePurchaseEnable)
        {
            $alreadyPurchased = $this->checkAlreadyPurchased($crmData);
            if ($alreadyPurchased)
            {
                $message = Config::get('addons.scheduled-blocker.display_message');
                return ['status' => false, 'api_response' => ['response_message' => $message]];
            }
        }

        $response = $this->callApi($method, $crmData);
        if ($this->skip_website_url)
        {
            $response['customer_add_note_response'] = $this->addCustomerNote($response);
        }
        else
        {
            $current_url                            = !empty($crmData['current_url']) ? $crmData['current_url'] : ((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https://" : "http://") . $_SERVER["SERVER_NAME"] . $_SERVER["REQUEST_URI"]);
            $response['customer_add_note_response'] = $this->addCustomerNote($response, $current_url);
        }
        return $response;
    }

    public function placeSplitOrder($request, $step)
    {

        $splitChargeDetails = Config::get(sprintf('offer_details.step%s.splitChargeDetails', $step));

        $selectedProducts = array_get($_SESSION, 'external_data.product_data', array_get($splitChargeDetails, 'selectedProduct'));

        $configData = [
            'selectedProducts' => $selectedProducts,
            'campaignId'       => array_get($_SESSION, 'external_data.campaign_id', array_get($splitChargeDetails, 'campaign_id')),
            'preAuthBillerId'  => array_get($splitChargeDetails, 'crm_force_gateway_id'),
            'shippingPrice'    => $this->shippingProfile->get('shipping_profile_price')
        ];

        if ($this->shippingProfile->get('shipping_profile_crm_id'))
        {
            $configData['shippingId'] = $this->shippingProfile->get('shipping_profile_crm_id');
        }

        return $this->newOrder($request, $configData, 'SplitOrder');
    }

    public function placeUpsellOrder($request, $step)
    {

        $offerConfig      = Config::get('offer_details.step' . $step);
        $selectedProducts = array_get($_SESSION, 'external_data.product_data', array_get($offerConfig, 'selectedProduct'));

        $configData = [
            'selectedProducts' => $selectedProducts,
            'campaignId'       => array_get($_SESSION, 'external_data.campaign_id', array_get($offerConfig, '.campaign_id')),
            'preAuthBillerId'  => array_get($offerConfig, 'crm_force_gateway_id'),
            'shippingPrice'    => $this->shippingProfile->get('shipping_profile_price')
        ];

        if ($this->shippingProfile->get('shipping_profile_crm_id'))
        {
            $configData['shippingId'] = $this->shippingProfile->get('shipping_profile_crm_id');
        }

        return $this->newOrder($request, $configData, 'UpsellOrder');
    }

    public function placeNewOrder($request, $step)
    {

        $offerConfig      = Config::get('offer_details.step' . $step);
        $selectedProducts = array_get($_SESSION, 'external_data.product_data', array_get($offerConfig, 'selectedProduct'));

        $configData = [
            'selectedProducts' => $selectedProducts,
            'campaignId'       => array_get($_SESSION, 'external_data.campaign_id', array_get($offerConfig, 'campaign_id')),
            'forceMerchantId'  => array_get($offerConfig, 'crm_force_gateway_id'),
            'shippingPrice'    => $this->shippingProfile->get('shipping_profile_price')
        ];

        if ($this->shippingProfile->get('shipping_profile_crm_id'))
        {
            $configData['shippingId'] = $this->shippingProfile->get('shipping_profile_crm_id');
        }

        return $this->newOrder($request, $configData, 'NewOrder');
    }

    public function newOrder($orderData, $configData, $type)
    {
        if ($type == 'SplitOrder' || $type == 'NewOrder' || $type == 'UpsellOrder')
        {
            if ($type == 'SplitOrder')
            {
                $method = 'NewOrderWithProspect';
            }
            else
            {
                $method = 'NewOrder';
            }

            $i = 1;

            foreach ($this->selectedProducts as $selectedProduct)
            {
                $orderData['product' . $i . '_id']           = $selectedProduct['crm_product_id'];
                $orderData['product' . $i . '_qty']          = $selectedProduct['crm_product_qty'];
                $orderData['product' . $i . '_price']        = $selectedProduct['crm_product_price'];
                $checkoutData['product' . $i . '_shipPrice'] = $selectedProduct['crm_product_shipping_price'];
                $i++;
            }

            $orderData['expyear']   = 20 . $orderData['expyear'];
            $orderData['paySource'] = !empty($orderData['paySource']) ? $orderData['paySource'] : 'CREDITCARD';

            $configData['campaignId']    = $this->campaignId;
            $configData['shipProfileId'] = $this->shippingProfile->get('shipping_profile_crm_id');
            $configData['salesUrl']      = $this->salesUrl;
            $configData['ipAddress']     = $this->ipAddress;

            unset($orderData['cc_type']);
            unset($configData['selectedProducts']);
            unset($configData['shippingPrice']);

            if (isset($configData['preAuthBillerId']) && $configData['preAuthBillerId'] == 0)
            {
                unset($configData['preAuthBillerId']);
            }

            $crmData = array_merge_recursive($orderData, $configData);
            $crmData = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
            $crmData = array_filter($crmData);

            $crmData['billShipSame'] = ($crmData['billShipSame'] == 'yes') ? 1 : 0;
        }

        $singlePurchaseEnable = Config::isSinglePurchaseEnable();
        if ($singlePurchaseEnable)
        {
            $alreadyPurchased = $this->checkAlreadyPurchased($crmData);
            if ($alreadyPurchased)
            {
                $message = Config::get('addons.scheduled-blocker.display_message');
                return ['status' => false, 'api_response' => ['response_message' => $message]];
            }
        }

        $response = $this->callApi($method, $crmData);
        if ($this->skip_website_url)
        {
            $response['customer_add_note_response'] = $this->addCustomerNote($response);
        }
        else
        {
            $current_url                            = !empty($crmData['current_url']) ? $crmData['current_url'] : ((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https://" : "http://") . $_SERVER["SERVER_NAME"] . $_SERVER["REQUEST_URI"]);
            $response['customer_add_note_response'] = $this->addCustomerNote($response, $current_url);
        }
        return $response;
    }

    public function orderView($queryStrings, $orderIds)
    {
        $orderData = [];

        foreach ($orderIds as $orderId)
        {
            if (empty($_SESSION['__orders__'][$orderId]))
            {
                continue;
            }

            $orderData[$orderId] = array(
                'productId'     => $_SESSION['__orders__'][$orderId][0]['productId'],
                'productName'   => $_SESSION['__orders__'][$orderId][0]['name'],
                'productPrice'  => $_SESSION['__orders__'][$orderId][0]['price'],
                'productQty'    => $_SESSION['__orders__'][$orderId][0]['qty'],
                'shippingPrice' => $_SESSION['__orders__'][$orderId][0]['shipping'],
            );
        }

        return $orderData;
    }

    public function validateCoupon($couponData)
    {
        $coupon_codes = Config::get('addons.coupon-promo-codes.coupon');

        $apiResponseArr = [];
        $status         = false;
        if (in_array($couponData['promo_code'], $coupon_codes))
        {
            $apiResponseArr['response_code'] = 100;
            $status                          = true;
        }
        else
        {
            $apiResponseArr['response_code'] = 430;
            $apiResponseArr['error_message'] = "Invalid Promo Code";
        }

        return ['status' => $status, 'api_response' => $apiResponseArr];
    }

    private function performConfirmOrder($orderId)
    {
        try
        {
            $url = $this->prepareEndpointApi($this->api_base_url, 'ConfirmOrder');
            $res = $this->httpRequest($url, array('orderId' => $orderId));
        }
        catch (\Exception $ex)
        {
            return false;
        }

        return true;
    }

    public function isDecline($response)
    {
        $decline = false;

        if(!empty($response['isOrderDecline'])) $decline = true;

        $decline_code = 'Transaction Declined';
        if (!empty($response['response_message']) && is_string($response['response_message']))
        {
            $decline = strpos($response['response_message'], $decline_code) !== false ? true : false;
        }
        return $decline;
    }

    public function singleOrderView($formData)
    {
        $method          = 'OrderDetails';
        $data['orderId'] = $formData['orderId'];
        $response        = $this->callApi($method, $data);
        if (!empty($response['api_response']['data'][0]))
        {
            $placeholder['order_id']           = !empty($response['api_response']['data'][0]['orderId']) ? $response['api_response']['data'][0]['orderId'] : '';
            $placeholder['email']              = !empty($response['api_response']['data'][0]['emailAddress']) ? $response['api_response']['data'][0]['emailAddress'] : '';
            $placeholder['billing_first_name'] = !empty($response['api_response']['data'][0]['firstName']) ? $response['api_response']['data'][0]['firstName'] : '';
            $placeholder['billing_last_name']  = !empty($response['api_response']['data'][0]['lastName']) ? $response['api_response']['data'][0]['lastName'] : '';

            return ['status' => 'success', 'apiResponse' => $response['api_response']['data'][0], 'placeholderToken' => $placeholder];
        }

        return ['status' => 'error', 'apiResponse' => $response['api_response']['response_message']];
    }

    public function hasOwnedThisOrder($data, $formData)
    {
        return (array_key_exists("orderId", $data['apiResponse']) && $data['apiResponse']['emailAddress'] == strtolower($formData['email'])) ? TRUE : FALSE;
    }

    public function prepareDataForCancel($data, $formData)
    {
        $formData['orderId'] = $data['apiResponse']['orderId'];
        return $formData;
    }

    public function cancelOrder($formData)
    {
        $method               = 'CancelOrder';
        $data['orderId']      = $formData['orderId'];
        $data['cancelReason'] = 'Customer Cancelled Order';
        $response             = $this->callApi($method, $data);
        if (!empty($response['status']))
        {
            return ['status' => 'success', 'apiResponse' => $apiResponseArr = $response['api_response']];
        }
        else
        {
            return ['status' => 'error', 'apiResponse' => $response['api_response']];
        }
    }

    // Add Customer Note Method starts
    public function addCustomerNote($response, $current_url = '')
    {
        $apiResponseMessage = 'The request couldn\'t be completed! Please try again later';
        if (!empty($response['api_response']['orderId']) && !empty($response['api_response']['customerId']))
        {
            $orderID    = $response['api_response']['orderId'];
            $customerId = $response['api_response']['customerId'];
            if (!empty($this->notes))
            {
                if (!empty($current_url))
                {
                    $notes = $this->notes . ' | ' . 'Order ID:# ' . $orderID . ' is placed from ' . $current_url;
                }
                else
                {
                    $notes = $this->notes . ' | ' . 'Order ID:# ' . $orderID;
                }
            }
            else
            {
                if (!empty($current_url))
                {
                    $notes = 'Order ID:# ' . $orderID . ' is placed from ' . $current_url;
                }
                else
                {
                    $notes = 'Order ID:# ' . $orderID;
                }
            }
            $addCustomerNoteUrl = $this->prepareEndpointApi($this->api_base_url, 'AddCustomerNote');
            $payload            = [
                'customerId' => $customerId,
                'message'    => $notes
            ];
            $request            = $this->httpRequest($addCustomerNoteUrl, $payload);
            $httpResponse       = $request->getStatusCode();
            if ($httpResponse == 200)
            {
                $response           = json_decode($request->getBody()->getContents(), true);
                $apiResponseMessage = !empty($response['message']) ? $response['message'] : $apiResponseMessage;
            }
            return $apiResponseMessage;
        }
    }

    /**
     * Checking order exist for an User using Card or Email
     * @param type $orderData
     * @return type Boolean
     */
    private function checkAlreadyPurchased($orderData)
    {
        $method            = 'OrderDetails';
        $userBlockSettings = Config::get('addons.scheduled-blocker');

        if($userBlockSettings['type'] === 'time') {
            $startDate  = date('m/d/Y', strtotime('-' . $userBlockSettings['type_value'] . 'Minutes'));
            $endDate   = date('m/d/Y');
        }
        if($userBlockSettings['type'] === 'date') {
            $startDate  = '01/01/2015';
            $endDate = date('m/d/Y', strtotime($userBlockSettings['type_value']));
        }

        if($startDate > $endDate) return false;

        if (!empty($orderData['emailAddress']) && !empty($userBlockSettings['block_type']))
        {
            $needToCheck = false;

            $curlPostData = [
                'startDate'    => $startDate,
                'endDate'      => $endDate,
                'orderStatus'  => 'COMPLETE',
            ];

            if(in_array('email', $userBlockSettings['block_type'])) {
                $curlPostData['emailAddress'] = $orderData['emailAddress'];
                $needToCheck = true;
            }

            if(in_array('ip', $userBlockSettings['block_type'])) {
                $curlPostData['ipAddress'] = !empty($orderData['ipAddress']) ? $orderData['ipAddress'] : $this->ipAddress;
                $needToCheck = true;
            }

            if(!$needToCheck) return false;

            $response = $this->callApi($method, $curlPostData);
            return (!empty($response['status']) && !empty($response['api_response']['totalResults']) && $response['api_response']['totalResults'] > 0) ? true : false;
        }
        return false;
    }

}
