<?php

namespace Framework\Modules;

use Framework\Foundation\Config;
use Framework\Foundation\Helper;
use GuzzleHttp\Client;

class CurrencyRates
{

    public static function updatRates()
    {
        $baseCurrency = Config::get('funnel.currency.code');
        if(!$baseCurrency) {
            throw new \Exception("Base currency not set!", AJAX_EX_CODE);
        }

        $currencyConverterApiKey = Config::get('addons.currency-converter.api_key');
        if (!$currencyConverterApiKey) {
            throw new \Exception("Empty currency list or currency converter api key!", AJAX_EX_CODE);
        }

        $url = sprintf('https://v6.exchangerate-api.com/v6/%s/latest/%s', $currencyConverterApiKey, $baseCurrency);
        $client = new Client();
        $responseArray = [];

        $currencyRatesFile      = STORAGE_PATH . DS . 'currency_rates.json';
        $guzzleResponse = json_decode($client->get($url)->getBody()->getContents(),true);

        if(empty($guzzleResponse['conversion_rates'])) {
            throw new \Exception("Unable to process!", AJAX_EX_CODE);
        }

        file_put_contents(
            $currencyRatesFile, json_encode(
                $guzzleResponse['conversion_rates']
            ), LOCK_EX
        );
        
        return pre_response_array(null, true, $guzzleResponse['conversion_rates']);
    }

    public static function updateToken(&$tokens, $countryCode)
    {
        if (empty(trim(Config::get('addons.currency-converter.api_key'))))
            return;

        $currencyFile        = STORAGE_PATH . DS . 'currency.json';
        $currencyRatesFile   = STORAGE_PATH . DS . 'currency_rates.json';
        $currencySymbolsFile = STORAGE_PATH . DS . 'currency_symbols.json';

        if (!is_readable($currencyFile) || !is_readable($currencyRatesFile) && !is_readable($currencySymbolsFile))
        {
            return;
        }

        $currencyRates          = Helper::getJsonFileData(STORAGE_PATH, 'currency_rates.json');
        $countryCurrencyMapping = Helper::getJsonFileData(STORAGE_PATH, 'currency.json');
        $symbolCurrencyMapping  = Helper::getJsonFileData(STORAGE_PATH, 'currency_symbols.json')->merge(
                Helper::getJsonFileData(CUSTOM_STORAGE_PATH, 'currency_symbols.json')
        );


        if (!empty($symbolCurrencyMapping[$countryCurrencyMapping[$countryCode]]))
        {
            $tokens['currency'] = html_entity_decode($symbolCurrencyMapping[$countryCurrencyMapping[$countryCode]]);
        }

        if (empty($countryCurrencyMapping[$countryCode]) || $countryCurrencyMapping[$countryCode] === 'USD')
        {
            return;
        }

        if (empty($currencyRates[$countryCurrencyMapping[$countryCode]]))
        {
            return;
        }

        $rate      = $currencyRates[$countryCurrencyMapping[$countryCode]];
        $tokenKeys = array_keys($tokens);
        $priceKeys = array_values(preg_grep('/price/', $tokenKeys));
        foreach ($priceKeys as $priceKey)
        {
            $tokens[$priceKey] = round($rate * $tokens[$priceKey], 2);
        }

    }

    /**
     * Check currency_rates.json for any update required or not.
     * @return boolean 
     */
    public static function isUpdateRequired()
    {
        $isRequired        = false;
        $currencyRatesFile = STORAGE_PATH . DS . 'currency_rates.json';
        if (!file_exists($currencyRatesFile) || (time() - filemtime($currencyRatesFile) > 12 * 60 * 60))
        {
            $currencyConverterApiKey = trim(Config::get('addons.currency-converter.api_key'));
            if (!empty($currencyConverterApiKey))
            {
                $isRequired = true;
            }
        }
        return $isRequired;
    }

}
