<?php

/**
 *
 * METHOD PARAMS SPECIFICATION.
 * $address[], Need to be a array which must have only 4 keys 'heading','addressLine1','addressLine2','addressLine3'

 * $uploadFolder, need to provide full path of the directory where image will be stored and the directory must be ended with '/' otherwise image will not store in the folder.

 * $lineFont and $headingFont need to provide the full path of the font with its font.ttf file.

 * $imgWidth and $imgHeight is need to provide (int value) otherwise it will take width = 400 , height = 300
 *
 * */

namespace Framework\Modules;

class ImageCreator
{

    public function createAddressToken($address = [], $uploadFolder = '', $lineFont = '', $headingFont = '', $imgWidth = '', $imgHeight = '')
    {

        //-----Createing a dynamic name for the image--------//
        $imageName = uniqid() . '.jpeg';

        //-------The Sample Address to draw------------------------//
        $addressArr = array(
            'heading'      => 'This is address',
            'addressLine1' => 'Dum Dum',
            'addressLine2' => 'Kolkata',
            'addressLine3' => 'pin - 700030',
        );

        //-------font path for address lines-------------------------------//
        $addresslineFont = dirname(dirname(__DIR__)) . DIRECTORY_SEPARATOR . 'storage' . DIRECTORY_SEPARATOR . 'fonts/HelveticaNeueLightCond.ttf';

        $line_font = $lineFont ? $lineFont : $addresslineFont;

        //-------font path for heading--------------//
        $HeadingFont = dirname(dirname(__DIR__)) . DIRECTORY_SEPARATOR . 'storage' . DIRECTORY_SEPARATOR . 'fonts/LatoBold.ttf';

        $heading_font = $headingFont ? $headingFont : $HeadingFont;

        //-------Upload directory path-----------//
        $uploadPath = dirname(dirname(__DIR__)) . DIRECTORY_SEPARATOR . 'storage' . DIRECTORY_SEPARATOR . 'images/' . $imageName;

        //------if upload directory is given---------//
        if ($uploadFolder != '')
        {
            $uploadFolder = $uploadFolder . $imageName;
        }

        //---------Setting the path conditionaly--------//
        $pathImage = $uploadFolder ? $uploadFolder : $uploadPath;

        //------Setting image height width conditionaly------//
        $width  = $imgWidth ? $imgWidth : 400;
        $height = $imgHeight ? $imgHeight : 300;

        //------Setting X and Y axis for the image---------//
        $x = 0;
        $y = 0;

        //------Create the image-------------//
        $im = imagecreatetruecolor($width, $height);

        //-------Create some colors for image background-------------------------//
        $white = imagecolorallocate($im, 255, 255, 255);
        $grey  = imagecolorallocate($im, 128, 128, 128);
        $black = imagecolorallocate($im, 0, 0, 0);
        imagefilledrectangle($im, $x, $y, $width, $height, $white);

        //-------If request address is provided or not setting heading and addresslines--------//

        if (!empty($address) && array_key_exists('heading', $address))
        {

            $addresHeading = $address['heading'];
        }
        else
        {
            $addresHeading = $addressArr['heading'];
        }

        if (!empty($address) && array_key_exists('addressLine1', $address))
        {

            $addresLin1 = $address['addressLine1'];
        }
        else
        {
            $addresLin1 = $addressArr['addressLine1'];
        }

        if (!empty($address) && array_key_exists('addressLine2', $address))
        {

            $addresLin2 = $address['addressLine2'];
        }
        else
        {
            $addresLin2 = $addressArr['addressLine2'];
        }

        if (!empty($address) && array_key_exists('addressLine3', $address))
        {

            $addresLin3 = $address['addressLine3'];
        }
        else
        {
            $addresLin3 = $addressArr['addressLine3'];
        }

        //--------Setting the heading in a appropiate position of the image with fonts-----//
        //----20 is font size----//
        imagettftext($im, 20, 0, 20, 100, $black, $heading_font, $addresHeading);

        //---------Setting the address lines in a appropiate position--------//
        //----14 is font size----//
        imagettftext($im, 14, 0, 20, 130, $black, $line_font, $addresLin1);

        //----14 is font size----//
        imagettftext($im, 14, 0, 20, 160, $black, $line_font, $addresLin2);

        //----14 is font size----//
        imagettftext($im, 14, 0, 20, 190, $black, $line_font, $addresLin3);

        //--------Add some shadow to the text------------//
        //imagettftext($im, 20, 0, 11, 21, $grey, $font, $text); always +1 from main image text
        //-----Out put the image and upload it into the folder------//
        imagepng($im, $pathImage);

        //-----Clearing the run time image creation memory----------//
        imagedestroy($im);

        return $pathImage;
    }

    public function createImage($txtName, $fontSize, $fontName, $hexColour)
    {

        $txtFont = '';

        //-------font path for text-------------------------------//
        $fontName = $fontName ? $fontName : 'arial';
        $fontName = str_replace("'", "", $fontName);

        $textFontdirectory = dirname(dirname(__DIR__)) . DIRECTORY_SEPARATOR . 'fonts' . DIRECTORY_SEPARATOR;

        $txtFontfile = dirname(dirname(__DIR__)) . DIRECTORY_SEPARATOR . 'fonts' . DIRECTORY_SEPARATOR . $fontName . '.ttf';

        //-------Checking if default font file exist o not, if not exist takeing the first appearing font.ttf file-----//
        if (file_exists($txtFontfile))
        {

            $txtFont = $txtFontfile;
        }
        else
        {
            $txtFont = glob($textFontdirectory . '*.ttf')[0];
        }

        //------Setting X and Y axis for the image---------//
        $x = 5;
        $y = 5;

        //------Font size-----------//
        $fontSize = $fontSize ? $fontSize : 20;

        //------Removeing single quote trom the text-----//
        $txtName = str_replace("'", "", $txtName);

        $array     = explode('\n', $txtName);
        $lengthArr = array();
        foreach ($array as $key)
        {

            $lenmgth = strlen($key);
            array_push($lengthArr, $lenmgth);
        }
        $maxLength = max($lengthArr);

        //---Get the key where max legth exist----//
        $key = array_search($maxLength, $lengthArr);

        //----The string which has maximum words------//
        $string = $array[$key];

        //------Createing the height width of the image by provideing the longest string--------------//
        $p         = imagettfbbox($fontSize, 0, $txtFont, $string);
        $imgWidth  = (abs($p[2] - $p[0]) + 10);
        $imgHeight = (abs($p[1] - $p[7]) + 10) * count($array);

        $oriHeight = abs($p[1] - $p[7]) + 10;

        //------Create the image-------------//
        $im = imagecreatetruecolor($imgWidth, $imgHeight);

        //-------Create some colors for image background-------------------------//
        $white = imagecolorallocate($im, 255, 255, 255);

        //--------Createing colour for the text---------------//
        $hexColour = str_replace("'", "", $hexColour);

        if ($hexColour != '')
        {
            $a = hexdec(substr($hexColour, 0, 2));
            $b = hexdec(substr($hexColour, 2, 2));
            $c = hexdec(substr($hexColour, 4, 2));
        }
        else
        {
            $a = 0;
            $b = 0;
            $c = 0;
        }
        $txtColour = imagecolorallocate($im, $a, $b, $c);

        //-------Putting the colour to image background---------//
        imagefilledrectangle($im, $x - 5, $y - 5, $imgWidth, $imgHeight, $white);

        //-------Putting the texts in the blank images with a loop------//
        foreach ($array as $key)
        {

            //-------Putting the text-----------------------------//
            imagettftext($im, $fontSize, 0, $x, $oriHeight - $y, $txtColour, $txtFont, $key);
            $y = ($y - ($fontSize + 5));
        }

        //-----Out put the image and upload it into the folder------//
        imagepng($im);

        //-----Clearing the run time image creation memory----------//
        imagedestroy($im);

        return true;
    }

}
