<?php

namespace Framework\Modules;

use Framework\Foundation\Config;
use GuzzleHttp\Client;

class PumpOneSSO
{

    public function __construct()
    {
        $this->has_pumpOneSso  = Config::get('enabled_PumpOneSSO');
        $this->pumpUpApiKey    = Config::get('pumpUpApiKey');
        $this->pumpUpApiSecret = Config::get('pumpUpApiSecret');
        $this->httpClient      = new Client();
        $this->httpClient->setDefaultOption('verify', false);
    }

    public function login($email, $user_id)
    {
        $token = '';
        if ($this->has_pumpOneSso && !empty($_SESSION['access_token']))
        {
            $data = [
                'email'   => $email,
                'user_id' => $user_id,
            ];

            $response     = $this->makeSsoRequest($data, 'login');
            $httpResponse = $response->getStatusCode();
            if ($httpResponse == 200)
            {
                $decoded_sso_response = json_decode($response->getBody()->getContents(), true);
                if (isset($decoded_sso_response['token']))
                {
                    $token                 = $decoded_sso_response['token'];
                    $_SESSION['sso_token'] = $token;
                }
            }
        }

        return $token;
    }

    public function removeAccess($userID)
    {
        $decoded_sso_response = '';
        $response             = $this->makeSsoRequest([
            'user_id' => $userID
                ], 'remove_access');
        $httpResponse         = $response->getStatusCode();
        if ($httpResponse == 200)
        {
            $decoded_sso_response = json_decode($response->getBody()->getContents(), true);
        }
        return $decoded_sso_response;
    }

    private function makeSsoRequest($userData, $method)
    {
        $url         = "https://www.pumpone.com/api/v1/sso/{$method}";
        $optionsData = [
            'api_key'           => $this->pumpUpApiKey,
            'api_secret'        => $this->pumpUpApiSecret,
            'subscription_type' => '1'
        ];
        $data        = array_merge($userData, $optionsData);

        $response = $this->httpClient->post($url, [
            'headers' => [
                'Accept'       => 'application/json',
                'Content-Type' => 'application/x-www-form-urlencoded'
            ],
            'body'    => $data
        ]);

        return $response;
    }

}
