<?php

namespace Framework\Modules;

use Framework\Foundation\Helper;
use Framework\Foundation\Config;
use Framework\Modules\PaayThreeDs;
use Framework\Modules\Crm\Adapter as CrmAdapter;
use Framework\Modules\Pixel;

class TemplateToken
{
    public function updateToken($request)
    {
        $token = [];
        if (Config::get('current_step') >= 1)
        {
            $currentStep = (int) Config::get('current_step');
            $currentPage = Config::get('current_page');

            $crmProductPrice      = $crmSplitProductPrice = 0.00;
            $shippingPrice        = $splitShippingPrice   = 0.00;
            $totalPrice           = $splitTotalPrice      = 0.00;

            $productsMap          = Config::get(sprintf('offer_details.step%s', $currentStep), []);
            $crmProductPrice      = array_get($productsMap, 'selectedProduct.0.crm_product_price', 0.00);
            $shippingPrice        = array_get($productsMap, 'shipping_price', 0.00);
            $crmSplitProductPrice = array_get($productsMap, 'splitChargeDetails.selectedProduct.0.crm_product_price', 0.00);
            $splitShippingPrice   = array_get($productsMap, 'splitChargeDetails.shipping_price', 0.00);

            $crmProductPrice      = empty($crmProductPrice) ? 0.00 : number_format($crmProductPrice, 2);
            $shippingPrice        = empty($shippingPrice) ? 0.00 : number_format($shippingPrice, 2);
            $crmSplitProductPrice = empty($crmSplitProductPrice) ? 0.00 : number_format($crmSplitProductPrice, 2);
            $splitShippingPrice   = empty($splitShippingPrice) ? 0.00 : number_format($splitShippingPrice, 2);


            $totalPrice      = $crmProductPrice + $shippingPrice;
            $splitTotalPrice = $crmSplitProductPrice + $splitShippingPrice;

            $billingDetail      = array_get($_SESSION, 'checkout_billing_data', '');
            $shippingDetail     = array_get($_SESSION, 'prospect_shipping_data', '');
            $countryCode        = array_get($_SESSION, 'prospect_shipping_data.country', '');
            $shippingStateCode  = array_get($_SESSION, 'prospect_shipping_data.state', '');
            $billingCountryCode = array_get($_SESSION, 'checkout_billing_data.billingCountry', '');
            $billingStateCode   = array_get($_SESSION, 'checkout_billing_data.billingState', '');
            $pixelInstance      = new Pixel();
            $getPixelInfo       = $pixelInstance->preparePixel('general');

            $token = [
                'app_ver'             => CMS_BAREBONE_VERSION,
                'funnel_name'         => Config::get('funnel.name'),
                'funnel_url'          => Helper::getValidFunnelUrl(),
                'app_base_url'        => app()->getBaseUrl($request),
                'funnel_path'         => Config::get('funnel.path'),
                'google_analytics_id' => Config::get('funnel.google_analytics_id'),
                'domain_name'              => Config::get('domain.name'),
                'copyright_name'           => Config::get('domain.copyright_name'),
                'address_inline'           => Config::get('domain.address_inline'),
                'address_formatted'        => Config::get('funnel.funnel_address_formatted'),
                'address_customer_support' => Config::get('domain.address_customer_support'),
                'address_other'            => Config::get('domain.address_other'),
                'address_return'           => Config::get('domain.address_return'),
                'domain_support_email'     => Config::get('domain.domain_support_email'),
                'domain_support_phone'     => Config::get('domain.domain_support_phone'),
                'notes'                    => Config::get('domain.notes'),
                'owner_first_name' => Config::get('owner.first_name'),
                'owner_last_name'  => Config::get('owner.last_name'),
                'owner_phone'      => Config::get('owner.phone'),
                'owner_email'      => Config::get('owner.email'),
                'company_name'           => Config::get('company.name'),
                'company_first_name'     => Config::get('company.first_name'),
                'company_second_name'    => Config::get('company.second_name'),
                'company_email_address'  => Config::get('company.email_address'),
                'company_address'        => Config::get('company.address'),
                'company_return_address' => Config::get('company.return_address'),
                'company_city'           => Config::get('company.city'),
                'company_state'          => Config::get('company.state'),
                'company_postal'         => Config::get('company.postal'),
                'company_phone'          => Config::get('company.phone'),
                'company_service_time'   => Config::get('company.service_time'),
                'shipping_price'       => $shippingPrice,
                'split_shipping_price' => $splitShippingPrice,
                'crm_product_price'    => $crmProductPrice,
                'split_product_price'     => $crmSplitProductPrice,
                'product_name'            => array_get($productsMap, 'selectedProduct.0.product_name'),
                'product_thumbnail_image' => array_get($productsMap, 'selectedProduct.0.product_thumbnail_image'),
                'product_original_image'  => array_get($productsMap, 'selectedProduct.0.product_original_image'),
                'crm_product_qty'         => array_get($productsMap, 'selectedProduct.0.crm_product_qty'),
                'crm_product_id'          => array_get($productsMap, 'selectedProduct.0.crm_product_id'),
                'insurance_provider_name'  => array_get($productsMap, 'splitChargeDetails.insurance_provider_name'),
                'insurance_provider_logo'  => array_get($productsMap, 'splitChargeDetails.insurance_provider_logo'),
                'shipping_insurance_price' => array_get($productsMap, 'shipping_insurance_price'),
                'credit_card_descriptor'   => array_get($productsMap, 'credit_card_descriptor'),
                'total_price'           => number_format($totalPrice, 2),
                'split_total_price'     => number_format($splitTotalPrice, 2),
                'shipping_country'      => Helper::getCountryName($countryCode),
                'shipping_country_code' => $countryCode,
                'shipping_state'        => Helper::getStateName($shippingStateCode),
                'shipping_state_code'   => $shippingStateCode,
                'shipping_first_name'   => array_get($shippingDetail, 'firstName', ''),
                'shipping_last_name'    => array_get($shippingDetail, 'lastName', ''),
                'shipping_address'      => array_get($shippingDetail, 'address', ''),
                'shipping_city'         => array_get($shippingDetail, 'city', ''),
                'shipping_zip'          => array_get($shippingDetail, 'zip', ''),
                'phone'                 => array_get($shippingDetail, 'phone', ''),
                'email'                 => array_get($shippingDetail, 'email', ''),
                'billing_first_name'    => array_get($billingDetail, 'billingFirstName', ''),
                'billing_last_name'     => array_get($billingDetail, 'billingLastName', ''),
                'billing_address'       => array_get($billingDetail, 'billingAddress', ''),
                'billing_city'          => array_get($billingDetail, 'billingCity', ''),
                'billing_country'       => Helper::getCountryName($billingCountryCode),
                'billing_country_code'  => $billingCountryCode,
                'billing_state'         => Helper::getStateName($billingStateCode),
                'billing_state_code'    => $billingStateCode,
                'billing_zip'           => array_get($billingDetail, 'billingZip', ''),
                'order_id'              => $request->query->get('order_id') ? $request->query->get('order_id') : '',
                'current_year'            => date('Y'),
                'widget'                  => 'widget.html',
                'order_receipt'           => 'order_receipt.html',
                'currency'                => html_entity_decode(Config::get('funnel.currency.symbol')),
                'disable_google_indexing' => $this->disableSearchIndexing(),
                'square_app_id'           => Config::get('payment_provider.api_password'),
                'enabled_PumpOneSSO'      => Config::get('enabled_PumpOneSSO'),
                'pumpUpApiKey'            => Config::get('pumpUpApiKey'),
                'pumpUpApiSecret'         => Config::get('pumpUpApiSecret'),
                'header_pixel_tag'        => html_entity_decode('<div id="header_tag_pixel" style="display:none;">' . $getPixelInfo['header_tag_pixel'] . '</div>'),
                'body_open_pixel_tag'     => html_entity_decode('<div id="body_start_tag_pixel" style="display:none;">' . $getPixelInfo['body_start_tag_pixel'] . '</div>'),
                'body_end_pixel_tag'      => html_entity_decode('<div id="body_end_tag_pixel" style="display:none;">' . $getPixelInfo['body_end_tag_pixel'] . '</div>'),
            ];
            $i = 1;
            foreach (Config::get('people', []) as $peopleData)
            {
                foreach ($peopleData as $key => $val)
                {
                    $token['people_' . $i . '_' . $key] = $val;
                }
                $i++;
            }
            if (!empty(Config::get('owner.custom_fields')))
            {
                foreach (Config::get('owner.custom_fields') as $key => $val)
                {
                    $token['owner_' . $key] = $val;
                }
            }
            if (!empty(Config::get('funnel.custom_fields')))
            {
                foreach (Config::get('funnel.custom_fields') as $key => $val)
                {
                    $token['funnel_' . $key] = $val;
                }
            }
            if (!empty(Config::get('company.custom_fields')))
            {
                foreach (Config::get('company.custom_fields') as $key => $val)
                {
                    $token['company_' . $key] = $val;
                }
            }

            /**
             * Set shipping profile value
             */

            if(!empty(Config::get('shipping_profile')))
            {
                $shipCount = 1;
                foreach(Config::get('shipping_profile') as $shipProfile)
                {
                    $token['shipping_profile_name_'.$shipCount]        = $shipProfile['shipping_profile_name'];
                    $token['shipping_profile_price_'.$shipCount]       = $shipProfile['shipping_profile_price'];
                    $token['shipping_profile_description_'.$shipCount] = $shipProfile['shipping_profile_description'];
                    $token['shipping_profile_crm_id_'.$shipCount]      = $shipProfile['shipping_profile_crm_id'];
                    $token['shipping_profile_provider_'.$shipCount]    = $shipProfile['shipping_profile_provider'];
                    if(!empty($shipProfile['custom_fields']))
                    {
                        foreach($shipProfile['custom_fields'] as $shipCustomkey => $shipCustomValue)
                        {
                            $token['shipping_profile_'.$shipCustomkey.'_'.$shipCount] = $shipCustomValue;
                        }
                    }
                    $shipCount++;
                }
            }

            /**
             * Set Global tokens value
             */
            if (!empty(Config::get('global_fields')))
            {
                foreach (Config::get('global_fields') as $key => $val)
                {
                    $token[$key] = $val;
                }
            }

            if (Config::get('addons.seo.google_indexing'))
            {
                $currency = Helper::getCurrencyByIp();
                if (!empty($currency['symbol']))
                {
                    $token['currency'] = $currency['symbol'];
                }
            }

            $company_service_hours          = Config::get('company.service_time');
            $token['company_service_hours'] = '';
            $token['company_service_time']  = Config::get('company.service_time');
            if (!empty($company_service_hours))
            {
                foreach ($company_service_hours as $i => $v)
                {
                    if ($i != 0)
                    {
                        if ($i == (count($company_service_hours) - 1))
                        {
                            $token['company_service_hours'] .= ' and';
                        }
                        else
                        {
                            $token['company_service_hours'] .= ', ';
                        }
                    }

                    foreach ($v as $ii => $item)
                    {
                        if ($ii == 'start_time')
                        {
                            $token['company_service_hours'] .= ', ';
                        }
                        elseif ($ii == 'end_time')
                        {
                            $token['company_service_hours'] .= ' - ';
                        }

                        $token['company_service_hours'] .= ' ' . $item;
                    }
                }
            }
            //Get variable tokens
            if (isset($_REQUEST))
            {
                $token['request'] = $_REQUEST;
            }

            $productToken = $this->getProductTokenVal();
            $token        = array_merge($productToken, $token);

            if (
                    Config::get('addons.currency-converter') &&
                    (!empty($_SESSION['current_country']) || !empty($_REQUEST['current_country']))
            )
            {
                CurrencyRates::updateToken($token, array_get($_REQUEST, 'current_country', !empty($_SESSION['current_country']) ? $_SESSION['current_country']:''));                
                if (!empty($_REQUEST['current_country']))
                {
                    $_SESSION['current_country'] = $_REQUEST['current_country'];
                }
            }
            $widget = Config::get('widget');
            if (PaayThreeDs::isActivePaay())
            {
                $widget['core_app_js_links'] .= '<script type="text/javascript" src="https://cdn.3dsintegrator.com/threeds-min-v1.6.0.js"></script>';
            }
            $widget = $this->reorderWidget($widget, $token);

            if (!empty($widget) && !empty($token))
            {
                $ArrayTemplateEngine = new \Framework\Foundation\TemplateEngine($widget);
                foreach ($widget as $key => $value)
                {
                    $token[$key] = html_entity_decode($ArrayTemplateEngine->compiler()->render($key, $token));
                }
            }

            $apiType = Config::get('payment_provider.api_type');

            if ($currentPage == 'thankyou')
            {
                if (empty($_SESSION['order_details']))
                {
                    $this->orderDetails($request);
                }

                $step = 1;
                if ($apiType == 'response')
                {
                    $orderKeys = preg_grep("/_order_id$/", array_keys($_SESSION));
                    foreach ($orderKeys as $orderKey)
                    {
                        $_SESSION[$orderKey] = (string) $_SESSION[$orderKey];
                    }
                }
                $token['total_price'] = 0;
                if (!empty($_SESSION['checkout_order_id']))
                {

                    $keyExist = isset($_SESSION['order_details'][(string) $request->query->get('order_id')]) ? true : false; //Response CRM index key
                    if (!array_key_exists($request->query->get('order_id'), $_SESSION['order_details']) && !$keyExist)
                    {
                        $orderDetails = $_SESSION['order_details'];
                    }
                    else
                    {
                        $orderDetails = $_SESSION['order_details'][$_SESSION['checkout_order_id']];
                    }

                    $step1_token          = [
                        'step1' => $this->prepareTokens($step, $orderDetails),
                    ];
                    $token['total_price'] += array_get($step1_token, 'step1.order_total', 0);

                    $token = array_merge_recursive($token, $step1_token);
                }

                if (
                        !empty($_SESSION['split_order_id']) &&
                        isset($_SESSION['order_details'][$_SESSION['split_order_id']]) &&
                        array_key_exists($_SESSION['split_order_id'], $_SESSION['order_details'])
                )
                {
                    $orderDetails         = $_SESSION['order_details'][$_SESSION['split_order_id']];
                    $split_token          = [
                        'split' => $this->prepareTokens($step, $orderDetails),
                    ];
                    $token['total_price'] += array_get($split_token, 'split.order_total', 0);
                    $token                = array_merge_recursive($token, $split_token);
                }

                if (!empty($_SESSION['upsell_order_ids']))
                {
                    $upsellArr      = [];
                    $tokenUpsellArr = [];

                    $upsellStep = count(Config::get('offer_details'));

                    for ($step_upsell = 2; $step_upsell <= $upsellStep; $step_upsell++)
                    {
                        foreach ($_SESSION['upsell_order_ids'] as $upsell)
                        {
                            if ($apiType == 'response')
                            {
                                $upsellArr[(string) $upsell] = $step_upsell;
                            }
                            else
                            {
                                $upsellArr[$upsell] = $step_upsell;
                            }
                        }
                    }

                    $i_upsell = 1;
                    $keyIndex = 0;
                    foreach ($upsellArr as $key_upsell => $val_upsell)
                    {
                        $orderDetails              = array_get($_SESSION, sprintf('order_details.%s', $key_upsell), []);
                        $tokenUpsellArr[$keyIndex] = array_merge($this->prepareTokens($val_upsell, $orderDetails), array('upsale_step' => $i_upsell . Helper::ordinalSuffix($i_upsell)));

                        if (!empty($_SESSION['upsell_split_order_ids']))
                        {
                            foreach ($_SESSION['upsell_split_order_ids'] as $key_upsell_split => $val_upsell_split)
                            {
                                if ($key_upsell_split == $key_upsell)
                                {
                                    $upsellSplitOrderDetails                   = array_get($_SESSION, sprintf('order_details.%s', $val_upsell_split[0]), []);
                                    $prepared_tokens                           = $this->prepareTokens($val_upsell_split[0], $upsellSplitOrderDetails);
                                    $token['total_price']                      += array_get($prepared_tokens, 'order_total', 0);
                                    $tokenUpsellArr[$keyIndex]['upsell_split'] = array_merge($prepared_tokens, array('upsale_step' => $i_upsell . Helper::ordinalSuffix($i_upsell)));
                                }
                            }
                        }

                        $i_upsell++;
                        $keyIndex++;
                    }

                    $upsell_token = [
                        'upsell' => $tokenUpsellArr,
                    ];

                    $token = array_merge_recursive($token, $upsell_token);
                }
            }
        }

        $token['total_price'] = number_format(array_get($token, 'total_price', 0), 2);

        $_SESSION['twig_tokens']      = $token;
        $token['show_google_captcha'] = UserAccess::_showGoogleCaptcha();
        return $token;
    }

    public function disableSearchIndexing() {
        $seo = Config::get('addons.seo');
         
        if(isset($seo['is_indexing'])) {
            return $seo['is_indexing'];
        }
        // Always return true (noindex) if the addon is not set
        return 1;
     }

    public function getProductTokenVal()
    {
        $offerDetails = Config::get('offer_details');
        $token        = [];
        $i            = 1;
        foreach ($offerDetails as $key => $val)
        {
            if ($key == 'step1')
            {
                $selectedProduct = isset($val['selectedProduct']) ? $val['selectedProduct'] : [];

                foreach ($selectedProduct as $pKey => $pVal)
                {
                    $postKey                                        = '_' . ($pKey + 1);
                    $token['product_name' . $postKey]               = $pVal['product_name'];
                    $token['elm_product_id' . $postKey]             = "elm_" . $pVal['product_id'];
                    $token['crm_product_id' . $postKey]             = $pVal['crm_product_id'];
                    $token['crm_product_qty' . $postKey]             = array_get($pVal, 'crm_product_qty', 1);

                    $token['crm_product_price' . $postKey]          = number_format((float) $pVal['crm_product_price'], 2, '.', '');

                    $token['product_original_image' . $postKey]     = array_get($pVal, 'product_original_image', '');
                    $token['product_thumbnail_image' . $postKey]    = array_get($pVal, 'product_thumbnail_image', '');

                    $token['split_product_price' . $postKey]        = number_format((float) array_get(
                        $offerDetails, $key.'.splitChargeDetails.selectedProduct.0.crm_product_price', '0.00'
                    ), 2, '.', '');

                    $token['product_rebill_price' . $postKey]      = array_get($pVal, 'product_rebill_price', '');
                    $token['product_trial_period' . $postKey]      = array_get($pVal, 'product_trial_period', '');
                    $token['product_shipping_period' . $postKey]   = array_get($pVal, 'product_shipping_period', '');
                    $token['product_evaluation_period' . $postKey] = array_get($pVal, 'product_evaluation_period', '');
                    $token['product_description' . $postKey]       = array_get($pVal, 'product_description', '');

                    if (!empty($pVal['custom_fields']))
                    {
                        foreach ($pVal['custom_fields'] as $prKey => $prVal)
                        {
                            $token['product' . '_' . $prKey . $postKey] = $prVal;
                        }
                    }

                    // These are special tokens which are added for legacy barebone support
                    $extraKeys  = [
                        'product_shipping_period',
                        'crm_product_shipping_price',
                        'crm_product_shipping_id',
                        'offer_id',
                        'billing_model_id',
                        'trial_product_id'
                    ];

                    if (!empty($pVal['extra_fields']))
                    {
                        foreach ($pVal['extra_fields'] as $exKey => $exVal)
                        {
                            if (!in_array($exKey, $extraKeys))
                            {
                                $token['ef' . '_' . $exKey . $postKey] = $exVal;
                            }else{
                                $token[$exKey . $postKey] = $exVal;
                            }
                        }
                    }
                }
            }
            else
            {
                if (substr($key, 0, 4) == "step")
                {
                    if (!empty($val['variation']))
                    {
                        foreach ($val['variation'] as $vKey => $vVal)
                        {
                            $selectedProduct = isset($vVal['selectedProduct']) ? $vVal['selectedProduct'] : [];
                            foreach ($selectedProduct as $pKey => $pVal)
                            {

                                $postKey = '_' . ($pKey + 1);
                                $indexName = 'upsell_' . $i . '_variant_' . ($vKey + 1);

                                $token[$indexName . '_product_name' . $postKey]               = $pVal['product_name'];
                                $token[$indexName . '_crm_product_id' . $postKey]             = $pVal['crm_product_id'];
                                $token[$indexName . '_product_id' . $postKey]                 = $pVal['product_id'];
                                $token[$indexName . '_crm_product_price' . $postKey]          = number_format( (float) $pVal['crm_product_price'], 2, '.', '');
                                $token[$indexName . '_product_original_image' . $postKey]     = array_get($pVal, 'product_original_image', '');
                                $token[$indexName . '_product_thumbnail_image' . $postKey]    = array_get($pVal, 'product_thumbnail_image', '');
                                $token[$indexName . '_crm_product_shipping_id' . $postKey]    = array_get($pVal, 'crm_product_shipping_id', '');
                                $token[$indexName . '_crm_product_shipping_price' . $postKey] = number_format(
                                    (float) array_get($pVal, 'crm_product_shipping_price', ''), 2, '.', ''
                                );

                                $token[$indexName . '_split_product_price' . $postKey] = number_format( 
                                    (float) array_get($token, $indexName . '._split_product_price.' . $postKey, 0.00), 2, '.', ''
                                );

                                $token[$indexName . '_product_rebill_price' . $postKey]      = number_format(
                                    (float) array_get($pVal, 'product_rebill_price', ''), 2, '.', ''
                                );

                                $token[$indexName . '_product_trial_period' . $postKey]      = array_get($pVal, 'product_trial_period', '');
                                $token[$indexName . '_product_shipping_period' . $postKey]   = array_get($pVal, 'product_shipping_period', '');
                                $token[$indexName . '_product_evaluation_period' . $postKey] = array_get($pVal, 'product_evaluation_period', '');
                                $token[$indexName . '_product_description' . $postKey]       = array_get($pVal, 'product_description', '');
                                if (!empty($pVal['custom_fields']))
                                {
                                    foreach ($pVal['custom_fields'] as $prKey => $prVal)
                                    {
                                        $token[$indexName . '_' . $prKey . $postKey] = $prVal;
                                    }
                                }
                            }
                        }
                    }
                }

                $i++;
            }
        }

        //manage custom tokens
        $customTokens = Config::get('addons.custom_tokens');

        if ($customTokens)
        {
            foreach ($customTokens as $cKey => $cVal)
            {
                foreach ($cVal as $key => $val)
                {
                    $token[$key] = $val;
                }
            }
        }
        //End of custom tokens
        return $token;
    }
 

    protected function orderDetails($request)
    {
        $queryStrings = [
            'prospect_id' => $request->query->get('prospect_id'),
            'customer_id' => $request->query->get('customer_id'),
            'order_id'    => $request->query->get('order_id'),
        ];

        $orderIds = [$request->query->get('order_id')];
        if (!empty($_SESSION['split_order_id']))
        {
            array_push($orderIds, $_SESSION['split_order_id']);
        }
        if (!empty($_SESSION['upsell_order_ids']))
        {
            $orderIds = array_merge_recursive($orderIds, $_SESSION['upsell_order_ids']);
        }
        if (!empty($_SESSION['upsell_split_order_ids']))
        {
            foreach ($_SESSION['upsell_split_order_ids'] as $key => $value)
            {
                $upsell_split_order_ids[] = $value[0];
            }

            $orderIds = array_merge_recursive($orderIds, $upsell_split_order_ids);
        }

        sort($orderIds);

        $crmAdapter                = new CrmAdapter(app()->crmInstance());
        $response                  = $crmAdapter->orderView($queryStrings, $orderIds);
        $_SESSION['order_details'] = $response;
    }

    protected function prepareTokens($step, $orderDetails)
    {
        $tokens   = [];
        $pruducts = [];

        $apiType = Config::get('payment_provider.api_type');

        if ($apiType == 'limelight' || $apiType == 'response' || $apiType == 'sublytics' || $apiType == 'vrio' || $apiType == 'twentyNineNext' || $apiType == 'stripe')
        {
            $sub_total = 0;

            if (!empty($orderDetails['products']))
            {
                foreach ($orderDetails['products'] as $product)
                {
                    $price       = !empty($product['price']) ? $product['price'] : 0.00;
                    $product_qty = !empty($product['product_qty']) ? $product['product_qty'] : 0;
                    $sub_total   = $sub_total + ($price * $product_qty);
                    $pruducts[]  = [
                        'product_name'  => !empty($product['name']) ? $product['name'] : 'None',
                        'product_price' => $price,
                        'product_qty'   => $product_qty,
                    ];
                }
            }

            $tokens = [
                'sub_total'      => $sub_total,
                'shipping_price' => Config::get('offer_details.step' . $step . '.shipping_price', 0.00),
                'sales_tax'      => !empty($orderDetails['order_sales_tax_amount']) ? $orderDetails['order_sales_tax_amount'] : 0.00,
                'order_date'     => !empty($orderDetails['acquisition_date']) ? date('d F, Y h:m:i a', strtotime($orderDetails['acquisition_date'])) : date('d F, Y h:m:i a'),
                'products'       => $pruducts,
            ];

            $tokens['order_total'] = $sub_total + $tokens['shipping_price'] + $tokens['sales_tax'];
        }
        else if ($apiType == 'konnektive')
        {
            $sub_total = 0;

            $price       = !empty($orderDetails['productPrice']) ? $orderDetails['productPrice'] : 0.00;
            $product_qty = !empty($orderDetails['productQty']) ? $orderDetails['productQty'] : 0;
            $sub_total   = $sub_total + ($price * $product_qty);

            $pruducts[] = [
                'product_name'  => !empty($orderDetails['productName']) ? $orderDetails['productName'] : 'None',
                'product_price' => $price,
                'product_qty'   => $product_qty,
            ];

            $tokens = [
                'sub_total'      => $sub_total,
                'shipping_price' => !empty($orderDetails['shippingPrice']) ? $orderDetails['shippingPrice'] : 0,
                'sales_tax'      => 0.00,
                'order_date'     => !empty($orderDetails['dateCreated']) ? date('d F, Y h:m:i a', strtotime($orderDetails['dateCreated'])) : date('d F, Y h:m:i a'),
                'products'       => $pruducts,
            ];

            $tokens['order_total'] = $sub_total + $tokens['shipping_price'] + $tokens['sales_tax'];
        }
        else if ($apiType == 'democrm')
        {
            $sub_total = 0;

            if (!empty($orderDetails['products']))
            {
                foreach ($orderDetails['products'] as $product)
                {
                    $price       = !empty($product['price']) ? $product['price'] : 0.00;
                    $product_qty = !empty($product['product_qty']) ? $product['product_qty'] : 0;
                    $sub_total   = $sub_total + ($price * $product_qty);

                    $pruducts[] = [
                        'product_name'  => !empty($product['name']) ? $product['name'] : 'None',
                        'product_price' => $price,
                        'product_qty'   => $product_qty,
                    ];
                }
            }

            $tokens = [
                'sub_total'      => $sub_total,
                'shipping_price' => Config::get('offer_details.step' . $step . '.shipping_price', 0.00),
                'sales_tax'      => !empty($orderDetails['order_sales_tax_amount']) ? $orderDetails['order_sales_tax_amount'] : 0.00,
                'order_date'     => !empty($orderDetails['acquisition_date']) ? date('d F, Y h:m:i a', strtotime($orderDetails['acquisition_date'])) : date('d F, Y h:m:i a'),
                'products'       => $pruducts,
            ];

            $tokens['order_total'] = $sub_total + $tokens['shipping_price'] + $tokens['sales_tax'];
        }

        return $tokens;
    }

    protected function getCurrentStep($page = '')
    {
        if ($page == 'index' || $page == 'checkout')
        {
            $currentStep = 1;
        }
        elseif ($page == 'upsell')
        {
            if (!isset($_SESSION['upsell_step']))
            {
                $currentStep = 2;
            }
            else
            {
                $currentStep = $_SESSION['upsell_step'];
            }
        }
        else
        {
            $currentStep = 1;
        }

        return $currentStep;
    }

    //reorder widget array based on hierarchy
    protected function reorderWidget($widget = [], $token = [])
    {
        $return_arr     = [];
        $alreadyScanned = [];
        foreach ($widget as $wi => $wv)
        {
            $return_arr = array_merge($return_arr, $this->scanWidgetForChildWidgets($widget, $token, $wi, $alreadyScanned));
        }

        return $return_arr;
    }

    protected function scanWidgetForChildWidgets($widget = [], $token = [], $wi, $alreadyScanned)
    {
        if (!isset($widget[$wi]))
        {
            return [];
        }

        $wv = $widget[$wi];

        //Add to already scanned
        $alreadyScanned[] = $wi;

        $widget_reordered = [];
        $output_array     = [];
        preg_match_all("/\[\[([^\]\]|^\s]*)(\s*\|\s*raw)*\]\]/", $wv, $output_array);
        if (empty($output_array[1]))
        {
            //No tokens inside, so it can be rendered safe
            $widget_reordered[$wi] = $wv;
            return $widget_reordered;
        }

        $child_widgets = [];
        $child_widgets = array_diff(array_unique(array_values($output_array[1])), array_merge(array_keys($token), $alreadyScanned));

        $return_arr = [];
        foreach ($child_widgets as $ci => $cv)
        {
            $return_arr = array_merge($return_arr, $this->scanWidgetForChildWidgets($widget, $token, $cv, $alreadyScanned));
        }
        return array_merge($return_arr, [$wi => $wv]);
    }
}
