<?php

namespace Framework;

class Routes
{

    public static $list = array(
        /**
         * Static Routes
         */
        '/'                           => 'StaticPageController@indexPage',
        '/static-assets'              => 'StaticPageController@directAccess',
        '/static-page'                => 'StaticPageController@openPages',
        '/checkout'                   => 'StaticPageController@checkoutPage',
        '/upsell'                     => 'StaticPageController@upsellPage',
        '/thank-you'                  => 'StaticPageController@thankYouPage',
        '/pages'                      => 'StaticPageController@staticPage',
        '/t35td@t@'                   => 'StaticPageController@testPage',
        '/image'                      => 'StaticPageController@generateImage',
        '/reset-validate'             => 'StaticPageController@resetValidation',
        '/pLanding'                   => 'StaticPageController@pumpOneLandingDashboard',
        '/limelight-3d-response'      => 'StaticPageController@limelight3DResponse',
        '/konnektive-3ds-response'    => 'StaticPageController@konnektive3dsResponse',
        '/mbilling-3ds-response'      => 'StaticPageController@Mbilling3dsResponse',
        '/paypal-response'             => 'StaticPageController@paypalResponse',

        /**
         * CRM ctl routes
         */
        '/ajax/prospect'              => 'CrmController@prospect',
        '/ajax/checkout'              => 'CrmController@checkout',
        '/ajax/offer-without-shipping'  => 'CrmController@offerWithoutShipping',
        '/ajax/offer'                 => 'CrmController@offer',
        '/ajax/upsell'                => 'CrmController@upsell',
        '/ajax/switch-upsell'         => 'CrmController@switchUpsell',
        '/ajax/validate'              => 'CrmController@validateCoupon',
        '/ajax/paay-three-ds'         => 'CrmController@paay3Ds',
        '/ajax/remote-upsell'         => 'CrmController@remoteUpsell',
        '/ll3dvalidation'             => 'CrmController@Ll3DValidation',
        '/ll3dpost'                   => 'CrmController@Ll3DPost',
        '/ajax/save-ll3d-data'        => 'CrmController@saveLL3dData',
        '/ajax/cancel'                => 'CrmController@orderCancel',
        '/ajax/client_secret'         => 'CrmController@clientSecret',

        /**
         * User ctl routes
         */
        '/ajax/access-register'       => 'UserController@accessRegister',
        '/ajax/access-login'          => 'UserController@accessLogin',
        '/ajax/access-logout'         => 'UserController@accessLogout',
        '/ajax/forget-password'       => 'UserController@forgetPassword',
        '/ajax/reset-password'        => 'UserController@resetPassword',
        '/ajax/post-mail'             => 'UserController@postMail',
        '/ajax/trigger'               => 'UserController@trigger',

        /**
         * Ajax Routes
         */
        '/ajax/update-currency-rates' => 'AjaxController@updateCurrencyRates',
        '/ajax/geolocation-data'      => 'AjaxController@geolocationData',
        '/ajax/fire-mail-chimp'       => 'AjaxController@fireMailChimp',
        '/ajax/config'                => 'AjaxController@config',
        '/ajax/cp-images'             => 'AjaxController@imageStorage',
        '/ajax/pixel'                 => 'AjaxController@getPixel',

        /**
         * External plugin Routes
         */
        '/external-plugin'            => 'ExternalPlugin@index',

        /**
         * Update config JSON Routes
         */
        '/update-config-json'        => 'CronController@updateConfigJson'
    );

    /**
     * The root route.json file data, we will override above list with the below list
     * @var $priorityList
     */
    public static $priorityList;

    /**
     * Check if the route path is restricted (auth required) or not
     * @param string $pathWithoutSlash
     * @return bool
     */
    public static function isProtected($pathWithoutSlash)
    {
        return array_get(static::$priorityList, sprintf('%s.restricted', $pathWithoutSlash)) == "true";
    }

    /**
     * Parse controller and method name from route path
     * @param string $path
     * @return array
     */
    public static function parseControllerMethod($path = '/')
    {

        $controller = null;
        $method     = null;

        if (array_get(static::$list, $path))
        {
            $ctrlMethod = explode('@', array_get(static::$list, $path));
            $controller = array_get($ctrlMethod, 0, null);
            $method     = array_get($ctrlMethod, 1, null);
        }

        return compact('controller', 'method');
    }

}
