<?php
// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Include PHPMailer files
require 'PHPMailer/src/Exception.php';
require 'PHPMailer/src/PHPMailer.php';
require 'PHPMailer/src/SMTP.php';

// Use statements for PHPMailer namespaces
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;

// Get raw POST data
$inputData = file_get_contents('php://input');

// Decode JSON data
$data = json_decode($inputData, true);

// Check if JSON data was successfully decoded
if (json_last_error() !== JSON_ERROR_NONE) {
    echo json_encode([
        'status' => 400,
        'message' => 'Invalid JSON format.'
    ]);
    exit;
}

// Extract data from the decoded JSON payload
$DBA_name = $data['DBA_name'] ?? '';
$cs_number = $data['cs_number'] ?? '';
$email = $data['email'] ?? '';
$mail_sub = $data['mail_sub'] ?? '';
$name = $data['name'] ?? '';
$order_id = $data['order_id'] ?? '';
$sender_mail = $data['sender_mail'] ?? '';
$smtp_token = $data['smtp_token'] ?? '';
$product_name = $data['product_name'] ?? '';
$reason = $data['reason'] ?? '';
$other_reason = $data['other_reason'] ?? '';

// Validate required fields
if (empty($email) || empty($smtp_token) || empty($sender_mail) || empty($DBA_name)) {
    echo json_encode([
        'status' => 400,
        'message' => 'Missing required fields.'
    ]);
    exit;
}

// Prepare the email body content
$bodyContent = "<h1>Thank you for contacting us!</h1>";
$bodyContent .= "<p>We have received your message and will get back to you as soon as possible.</p>";
$bodyContent .= "<p>Here is the information you submitted:</p>";
$bodyContent .= "<ul>";
$bodyContent .= "<li><strong>Name:</strong> $name</li>";
$bodyContent .= "<li><strong>Your Order Id:</strong> $order_id</li>";
$bodyContent .= "<li><strong>Product Name:</strong> $product_name</li>";
$bodyContent .= "<li><strong>Reason:</strong> $reason</li>";
if ($other_reason != '' && !empty($other_reason)) {
    $bodyContent .= "<li><strong>Other Reason:</strong> $other_reason</li>";
}
$bodyContent .= "</ul>";
$bodyContent .= "<p>Best regards,<br>$DBA_name Team</p>";
$bodyContent .= "<p>Contact us at <a href='tel:$cs_number'>$cs_number</a> or <a href='mailto:$sender_mail'>$sender_mail</a>.</p>";

// Create a new PHPMailer instance
$mail = new PHPMailer(true); // Enable exceptions

try {
    // Server settings
    $mail->isSMTP();
    $mail->SMTPDebug = SMTP::DEBUG_SERVER; // Enable verbose debug output
    $mail->SMTPAuth = true;
    $mail->Host = 'smtp.postmarkapp.com';
    $mail->Username = $smtp_token;
    $mail->Password = $smtp_token;
    $mail->SMTPSecure = 'tls';
    $mail->Port = 587;

    // Recipients
    $mail->setFrom($sender_mail, $DBA_name);
    $mail->addReplyTo($sender_mail);
    $mail->addAddress($email);
    $mail->addBCC("rosek.pas@gmail.com");
    // $mail->addBCC("danielg@digitalmvmt.com");
    // $mail->addBCC("tiffany@digitalmvmt.com");

    // Content
    $mail->isHTML(true);
    $mail->Subject = $mail_sub;
    $mail->Body = $bodyContent;

    $mail->send();
    echo json_encode([
        'status' => 200,
        'message' => 'Contact received successfully.'
    ]);
} catch (Exception $e) {
    // Log the error for debugging
    error_log("Mailer Error: " . $mail->ErrorInfo);

    echo json_encode([
        'status' => 500,
        'message' => 'Email sending failed. Please try again later.'
    ]);
}
?>
