<?php

/**
 * @version 1.0.0
 * Server Details Script to extract modules, disk space, and file usage limits in percentage
 */
error_reporting(1);
@ini_set('display_errors', 1);
set_time_limit(3600);

class ServerDetailsScript {

    public function __construct() {
        $this->handleRequest();
    }

    private function handleRequest() {
        if ($_SERVER['REQUEST_METHOD'] === 'GET') {
            $this->getDetails();
        } else {
            header("HTTP/1.0 405 Method Not Allowed");
            $this->json(["error" => "Method not allowed"]);
        }
    }

    /**
     * Function to get details including PHP version, server settings, available modules, disk space, and file usage limits
     */
    private function getDetails() {
        $phpVersion = phpversion();
        $maxExecutionTime = ini_get('max_execution_time');
        $defaultSocketTimeout = ini_get('default_socket_timeout');
        $module = [];
        $command = [];
        
        // Check available modules
        $commandExists = function($command) {
            $output = [];
            $returnVar = 0;
            exec("command -v $command", $output, $returnVar);
            return $returnVar === 0;
        };
        
        if (function_exists('exec')) array_push($module, "exec");
        if (function_exists('system')) array_push($module, "system");
        if (class_exists('ZipArchive')) array_push($module, "ZipArchive");
        if (class_exists('PclZip')) array_push($module, "PclZip");
        if ($commandExists('unzip')) array_push($command, "unzip");

        $rootDir = '/';
        $freeSpaceMB = round(disk_free_space($rootDir) / 1048576, 2);
        $totalSpaceMB = round(disk_total_space($rootDir) / 1048576, 2);

        $this->json([
            "success" => true,
            "data" => [
                'phpVersion' => $phpVersion,
                'max_execution_time' => $maxExecutionTime,
                'default_socket_timeout' => $defaultSocketTimeout,
                'module' => $module,
                'command' => $command,
                'disk_space' => [
                    'free_space_mb' => $freeSpaceMB,
                    'total_space_mb' => $totalSpaceMB
                ]
            ]
        ]);
    }

    /**
     * JSON response helper function
     */
    private function json($response) {
        header('Content-Type: application/json');
        echo json_encode($response);
        die();
    }
}

// Instantiate the ServerDetailsScript class to trigger the logic
new ServerDetailsScript();

?>
